import java.io
import java.lang
import java.net
import java.nio
import java.nio.channels.spi
import java.nio.charset
import java.nio.file
import java.nio.file.attribute
import java.util
import java.util.concurrent
import java.util.function
import jpype.protocol
import typing



class AcceptPendingException(java.lang.IllegalStateException):
    def __init__(self): ...

class AlreadyBoundException(java.lang.IllegalStateException):
    def __init__(self): ...

class AlreadyConnectedException(java.lang.IllegalStateException):
    def __init__(self): ...

class AsynchronousChannelGroup:
    def awaitTermination(self, long: int, timeUnit: java.util.concurrent.TimeUnit) -> bool: ...
    def isShutdown(self) -> bool: ...
    def isTerminated(self) -> bool: ...
    def provider(self) -> java.nio.channels.spi.AsynchronousChannelProvider: ...
    def shutdown(self) -> None: ...
    def shutdownNow(self) -> None: ...
    @staticmethod
    def withCachedThreadPool(executorService: java.util.concurrent.ExecutorService, int: int) -> 'AsynchronousChannelGroup': ...
    @staticmethod
    def withFixedThreadPool(int: int, threadFactory: java.util.concurrent.ThreadFactory) -> 'AsynchronousChannelGroup': ...
    @staticmethod
    def withThreadPool(executorService: java.util.concurrent.ExecutorService) -> 'AsynchronousChannelGroup': ...

class CancelledKeyException(java.lang.IllegalStateException):
    def __init__(self): ...

class Channel(java.io.Closeable):
    def close(self) -> None: ...
    def isOpen(self) -> bool: ...

class Channels:
    @typing.overload
    @staticmethod
    def newChannel(inputStream: java.io.InputStream) -> 'ReadableByteChannel': ...
    @typing.overload
    @staticmethod
    def newChannel(outputStream: java.io.OutputStream) -> 'WritableByteChannel': ...
    @typing.overload
    @staticmethod
    def newInputStream(asynchronousByteChannel: 'AsynchronousByteChannel') -> java.io.InputStream: ...
    @typing.overload
    @staticmethod
    def newInputStream(readableByteChannel: 'ReadableByteChannel') -> java.io.InputStream: ...
    @typing.overload
    @staticmethod
    def newOutputStream(asynchronousByteChannel: 'AsynchronousByteChannel') -> java.io.OutputStream: ...
    @typing.overload
    @staticmethod
    def newOutputStream(writableByteChannel: 'WritableByteChannel') -> java.io.OutputStream: ...
    @typing.overload
    @staticmethod
    def newReader(readableByteChannel: 'ReadableByteChannel', string: str) -> java.io.Reader: ...
    @typing.overload
    @staticmethod
    def newReader(readableByteChannel: 'ReadableByteChannel', charset: java.nio.charset.Charset) -> java.io.Reader: ...
    @typing.overload
    @staticmethod
    def newReader(readableByteChannel: 'ReadableByteChannel', charsetDecoder: java.nio.charset.CharsetDecoder, int: int) -> java.io.Reader: ...
    @typing.overload
    @staticmethod
    def newWriter(writableByteChannel: 'WritableByteChannel', string: str) -> java.io.Writer: ...
    @typing.overload
    @staticmethod
    def newWriter(writableByteChannel: 'WritableByteChannel', charset: java.nio.charset.Charset) -> java.io.Writer: ...
    @typing.overload
    @staticmethod
    def newWriter(writableByteChannel: 'WritableByteChannel', charsetEncoder: java.nio.charset.CharsetEncoder, int: int) -> java.io.Writer: ...

class ClosedChannelException(java.io.IOException):
    def __init__(self): ...

class ClosedSelectorException(java.lang.IllegalStateException):
    def __init__(self): ...

_CompletionHandler__V = typing.TypeVar('_CompletionHandler__V')  # <V>
_CompletionHandler__A = typing.TypeVar('_CompletionHandler__A')  # <A>
class CompletionHandler(typing.Generic[_CompletionHandler__V, _CompletionHandler__A]):
    def completed(self, v: _CompletionHandler__V, a: _CompletionHandler__A) -> None: ...
    def failed(self, throwable: java.lang.Throwable, a: _CompletionHandler__A) -> None: ...

class ConnectionPendingException(java.lang.IllegalStateException):
    def __init__(self): ...

class FileLock(java.lang.AutoCloseable):
    def acquiredBy(self) -> Channel: ...
    def channel(self) -> 'FileChannel': ...
    def close(self) -> None: ...
    def isShared(self) -> bool: ...
    def isValid(self) -> bool: ...
    def overlaps(self, long: int, long2: int) -> bool: ...
    def position(self) -> int: ...
    def release(self) -> None: ...
    def size(self) -> int: ...
    def toString(self) -> str: ...

class FileLockInterruptionException(java.io.IOException):
    def __init__(self): ...

class IllegalBlockingModeException(java.lang.IllegalStateException):
    def __init__(self): ...

class IllegalChannelGroupException(java.lang.IllegalArgumentException):
    def __init__(self): ...

class IllegalSelectorException(java.lang.IllegalArgumentException):
    def __init__(self): ...

class InterruptedByTimeoutException(java.io.IOException):
    def __init__(self): ...

class MembershipKey:
    def block(self, inetAddress: java.net.InetAddress) -> 'MembershipKey': ...
    def channel(self) -> 'MulticastChannel': ...
    def drop(self) -> None: ...
    def group(self) -> java.net.InetAddress: ...
    def isValid(self) -> bool: ...
    def networkInterface(self) -> java.net.NetworkInterface: ...
    def sourceAddress(self) -> java.net.InetAddress: ...
    def unblock(self, inetAddress: java.net.InetAddress) -> 'MembershipKey': ...

class NoConnectionPendingException(java.lang.IllegalStateException):
    def __init__(self): ...

class NonReadableChannelException(java.lang.IllegalStateException):
    def __init__(self): ...

class NonWritableChannelException(java.lang.IllegalStateException):
    def __init__(self): ...

class NotYetBoundException(java.lang.IllegalStateException):
    def __init__(self): ...

class NotYetConnectedException(java.lang.IllegalStateException):
    def __init__(self): ...

class OverlappingFileLockException(java.lang.IllegalStateException):
    def __init__(self): ...

class ReadPendingException(java.lang.IllegalStateException):
    def __init__(self): ...

class SelectionKey:
    OP_READ: typing.ClassVar[int] = ...
    OP_WRITE: typing.ClassVar[int] = ...
    OP_CONNECT: typing.ClassVar[int] = ...
    OP_ACCEPT: typing.ClassVar[int] = ...
    def attach(self, object: typing.Any) -> typing.Any: ...
    def attachment(self) -> typing.Any: ...
    def cancel(self) -> None: ...
    def channel(self) -> 'SelectableChannel': ...
    @typing.overload
    def interestOps(self) -> int: ...
    @typing.overload
    def interestOps(self, int: int) -> 'SelectionKey': ...
    def interestOpsAnd(self, int: int) -> int: ...
    def interestOpsOr(self, int: int) -> int: ...
    def isAcceptable(self) -> bool: ...
    def isConnectable(self) -> bool: ...
    def isReadable(self) -> bool: ...
    def isValid(self) -> bool: ...
    def isWritable(self) -> bool: ...
    def readyOps(self) -> int: ...
    def selector(self) -> 'Selector': ...

class Selector(java.io.Closeable):
    def close(self) -> None: ...
    def isOpen(self) -> bool: ...
    def keys(self) -> java.util.Set[SelectionKey]: ...
    @staticmethod
    def open() -> 'Selector': ...
    def provider(self) -> java.nio.channels.spi.SelectorProvider: ...
    @typing.overload
    def select(self) -> int: ...
    @typing.overload
    def select(self, long: int) -> int: ...
    @typing.overload
    def select(self, consumer: typing.Union[java.util.function.Consumer[SelectionKey], typing.Callable[[SelectionKey], None]]) -> int: ...
    @typing.overload
    def select(self, consumer: typing.Union[java.util.function.Consumer[SelectionKey], typing.Callable[[SelectionKey], None]], long: int) -> int: ...
    @typing.overload
    def selectNow(self) -> int: ...
    @typing.overload
    def selectNow(self, consumer: typing.Union[java.util.function.Consumer[SelectionKey], typing.Callable[[SelectionKey], None]]) -> int: ...
    def selectedKeys(self) -> java.util.Set[SelectionKey]: ...
    def wakeup(self) -> 'Selector': ...

class ShutdownChannelGroupException(java.lang.IllegalStateException):
    def __init__(self): ...

class UnresolvedAddressException(java.lang.IllegalArgumentException):
    def __init__(self): ...

class UnsupportedAddressTypeException(java.lang.IllegalArgumentException):
    def __init__(self): ...

class WritePendingException(java.lang.IllegalStateException):
    def __init__(self): ...

class AsynchronousChannel(Channel):
    def close(self) -> None: ...

class AsynchronousCloseException(ClosedChannelException):
    def __init__(self): ...

class InterruptibleChannel(Channel):
    def close(self) -> None: ...

class NetworkChannel(Channel):
    def bind(self, socketAddress: java.net.SocketAddress) -> 'NetworkChannel': ...
    def getLocalAddress(self) -> java.net.SocketAddress: ...
    _getOption__T = typing.TypeVar('_getOption__T')  # <T>
    def getOption(self, socketOption: java.net.SocketOption[_getOption__T]) -> _getOption__T: ...
    _setOption__T = typing.TypeVar('_setOption__T')  # <T>
    def setOption(self, socketOption: java.net.SocketOption[_setOption__T], t: _setOption__T) -> 'NetworkChannel': ...
    def supportedOptions(self) -> java.util.Set[java.net.SocketOption[typing.Any]]: ...

class ReadableByteChannel(Channel):
    def read(self, byteBuffer: java.nio.ByteBuffer) -> int: ...

class SelectableChannel(java.nio.channels.spi.AbstractInterruptibleChannel, Channel):
    def blockingLock(self) -> typing.Any: ...
    def configureBlocking(self, boolean: bool) -> 'SelectableChannel': ...
    def isBlocking(self) -> bool: ...
    def isRegistered(self) -> bool: ...
    def keyFor(self, selector: Selector) -> SelectionKey: ...
    def provider(self) -> java.nio.channels.spi.SelectorProvider: ...
    @typing.overload
    def register(self, selector: Selector, int: int, object: typing.Any) -> SelectionKey: ...
    @typing.overload
    def register(self, selector: Selector, int: int) -> SelectionKey: ...
    def validOps(self) -> int: ...

class WritableByteChannel(Channel):
    def write(self, byteBuffer: java.nio.ByteBuffer) -> int: ...

class AsynchronousByteChannel(AsynchronousChannel):
    _read_1__A = typing.TypeVar('_read_1__A')  # <A>
    @typing.overload
    def read(self, byteBuffer: java.nio.ByteBuffer) -> java.util.concurrent.Future[int]: ...
    @typing.overload
    def read(self, byteBuffer: java.nio.ByteBuffer, a: _read_1__A, completionHandler: CompletionHandler[int, _read_1__A]) -> None: ...
    _write_1__A = typing.TypeVar('_write_1__A')  # <A>
    @typing.overload
    def write(self, byteBuffer: java.nio.ByteBuffer) -> java.util.concurrent.Future[int]: ...
    @typing.overload
    def write(self, byteBuffer: java.nio.ByteBuffer, a: _write_1__A, completionHandler: CompletionHandler[int, _write_1__A]) -> None: ...

class AsynchronousFileChannel(AsynchronousChannel):
    def force(self, boolean: bool) -> None: ...
    _lock_1__A = typing.TypeVar('_lock_1__A')  # <A>
    _lock_3__A = typing.TypeVar('_lock_3__A')  # <A>
    @typing.overload
    def lock(self, long: int, long2: int, boolean: bool) -> java.util.concurrent.Future[FileLock]: ...
    @typing.overload
    def lock(self, long: int, long2: int, boolean: bool, a: _lock_1__A, completionHandler: CompletionHandler[FileLock, _lock_1__A]) -> None: ...
    @typing.overload
    def lock(self) -> java.util.concurrent.Future[FileLock]: ...
    @typing.overload
    def lock(self, a: _lock_3__A, completionHandler: CompletionHandler[FileLock, _lock_3__A]) -> None: ...
    @typing.overload
    @staticmethod
    def open(path: typing.Union[java.nio.file.Path, jpype.protocol.SupportsPath], openOptionArray: typing.List[java.nio.file.OpenOption]) -> 'AsynchronousFileChannel': ...
    @typing.overload
    @staticmethod
    def open(path: typing.Union[java.nio.file.Path, jpype.protocol.SupportsPath], set: java.util.Set[java.nio.file.OpenOption], executorService: java.util.concurrent.ExecutorService, fileAttributeArray: typing.List[java.nio.file.attribute.FileAttribute[typing.Any]]) -> 'AsynchronousFileChannel': ...
    _read_1__A = typing.TypeVar('_read_1__A')  # <A>
    @typing.overload
    def read(self, byteBuffer: java.nio.ByteBuffer, long: int) -> java.util.concurrent.Future[int]: ...
    @typing.overload
    def read(self, byteBuffer: java.nio.ByteBuffer, long: int, a: _read_1__A, completionHandler: CompletionHandler[int, _read_1__A]) -> None: ...
    def size(self) -> int: ...
    def truncate(self, long: int) -> 'AsynchronousFileChannel': ...
    @typing.overload
    def tryLock(self, long: int, long2: int, boolean: bool) -> FileLock: ...
    @typing.overload
    def tryLock(self) -> FileLock: ...
    _write_1__A = typing.TypeVar('_write_1__A')  # <A>
    @typing.overload
    def write(self, byteBuffer: java.nio.ByteBuffer, long: int) -> java.util.concurrent.Future[int]: ...
    @typing.overload
    def write(self, byteBuffer: java.nio.ByteBuffer, long: int, a: _write_1__A, completionHandler: CompletionHandler[int, _write_1__A]) -> None: ...

class AsynchronousServerSocketChannel(AsynchronousChannel, NetworkChannel):
    _accept_1__A = typing.TypeVar('_accept_1__A')  # <A>
    @typing.overload
    def accept(self) -> java.util.concurrent.Future['AsynchronousSocketChannel']: ...
    @typing.overload
    def accept(self, a: _accept_1__A, completionHandler: CompletionHandler['AsynchronousSocketChannel', _accept_1__A]) -> None: ...
    @typing.overload
    def bind(self, socketAddress: java.net.SocketAddress, int: int) -> 'AsynchronousServerSocketChannel': ...
    @typing.overload
    def bind(self, socketAddress: java.net.SocketAddress) -> 'AsynchronousServerSocketChannel': ...
    def getLocalAddress(self) -> java.net.SocketAddress: ...
    @typing.overload
    @staticmethod
    def open() -> 'AsynchronousServerSocketChannel': ...
    @typing.overload
    @staticmethod
    def open(asynchronousChannelGroup: AsynchronousChannelGroup) -> 'AsynchronousServerSocketChannel': ...
    def provider(self) -> java.nio.channels.spi.AsynchronousChannelProvider: ...
    _setOption__T = typing.TypeVar('_setOption__T')  # <T>
    def setOption(self, socketOption: java.net.SocketOption[_setOption__T], t: _setOption__T) -> 'AsynchronousServerSocketChannel': ...

class ByteChannel(ReadableByteChannel, WritableByteChannel): ...

class ClosedByInterruptException(AsynchronousCloseException):
    def __init__(self): ...

class GatheringByteChannel(WritableByteChannel):
    @typing.overload
    def write(self, byteBuffer: java.nio.ByteBuffer) -> int: ...
    @typing.overload
    def write(self, byteBufferArray: typing.List[java.nio.ByteBuffer]) -> int: ...
    @typing.overload
    def write(self, byteBufferArray: typing.List[java.nio.ByteBuffer], int: int, int2: int) -> int: ...

class MulticastChannel(NetworkChannel):
    def close(self) -> None: ...
    @typing.overload
    def join(self, inetAddress: java.net.InetAddress, networkInterface: java.net.NetworkInterface) -> MembershipKey: ...
    @typing.overload
    def join(self, inetAddress: java.net.InetAddress, networkInterface: java.net.NetworkInterface, inetAddress2: java.net.InetAddress) -> MembershipKey: ...

class ScatteringByteChannel(ReadableByteChannel):
    @typing.overload
    def read(self, byteBuffer: java.nio.ByteBuffer) -> int: ...
    @typing.overload
    def read(self, byteBufferArray: typing.List[java.nio.ByteBuffer]) -> int: ...
    @typing.overload
    def read(self, byteBufferArray: typing.List[java.nio.ByteBuffer], int: int, int2: int) -> int: ...

class ServerSocketChannel(java.nio.channels.spi.AbstractSelectableChannel, NetworkChannel):
    def accept(self) -> 'SocketChannel': ...
    @typing.overload
    def bind(self, socketAddress: java.net.SocketAddress, int: int) -> 'ServerSocketChannel': ...
    @typing.overload
    def bind(self, socketAddress: java.net.SocketAddress) -> 'ServerSocketChannel': ...
    def getLocalAddress(self) -> java.net.SocketAddress: ...
    @staticmethod
    def open() -> 'ServerSocketChannel': ...
    _setOption__T = typing.TypeVar('_setOption__T')  # <T>
    def setOption(self, socketOption: java.net.SocketOption[_setOption__T], t: _setOption__T) -> 'ServerSocketChannel': ...
    def socket(self) -> java.net.ServerSocket: ...
    def validOps(self) -> int: ...

class AsynchronousSocketChannel(AsynchronousByteChannel, NetworkChannel):
    def bind(self, socketAddress: java.net.SocketAddress) -> 'AsynchronousSocketChannel': ...
    _connect_1__A = typing.TypeVar('_connect_1__A')  # <A>
    @typing.overload
    def connect(self, socketAddress: java.net.SocketAddress) -> java.util.concurrent.Future[None]: ...
    @typing.overload
    def connect(self, socketAddress: java.net.SocketAddress, a: _connect_1__A, completionHandler: CompletionHandler[None, _connect_1__A]) -> None: ...
    def getLocalAddress(self) -> java.net.SocketAddress: ...
    def getRemoteAddress(self) -> java.net.SocketAddress: ...
    @typing.overload
    @staticmethod
    def open() -> 'AsynchronousSocketChannel': ...
    @typing.overload
    @staticmethod
    def open(asynchronousChannelGroup: AsynchronousChannelGroup) -> 'AsynchronousSocketChannel': ...
    def provider(self) -> java.nio.channels.spi.AsynchronousChannelProvider: ...
    _read_1__A = typing.TypeVar('_read_1__A')  # <A>
    _read_2__A = typing.TypeVar('_read_2__A')  # <A>
    _read_3__A = typing.TypeVar('_read_3__A')  # <A>
    @typing.overload
    def read(self, byteBuffer: java.nio.ByteBuffer) -> java.util.concurrent.Future[int]: ...
    @typing.overload
    def read(self, byteBuffer: java.nio.ByteBuffer, long: int, timeUnit: java.util.concurrent.TimeUnit, a: _read_1__A, completionHandler: CompletionHandler[int, _read_1__A]) -> None: ...
    @typing.overload
    def read(self, byteBufferArray: typing.List[java.nio.ByteBuffer], int: int, int2: int, long: int, timeUnit: java.util.concurrent.TimeUnit, a: _read_2__A, completionHandler: CompletionHandler[int, _read_2__A]) -> None: ...
    @typing.overload
    def read(self, byteBuffer: java.nio.ByteBuffer, a: _read_3__A, completionHandler: CompletionHandler[int, _read_3__A]) -> None: ...
    _setOption__T = typing.TypeVar('_setOption__T')  # <T>
    def setOption(self, socketOption: java.net.SocketOption[_setOption__T], t: _setOption__T) -> 'AsynchronousSocketChannel': ...
    def shutdownInput(self) -> 'AsynchronousSocketChannel': ...
    def shutdownOutput(self) -> 'AsynchronousSocketChannel': ...
    _write_1__A = typing.TypeVar('_write_1__A')  # <A>
    _write_2__A = typing.TypeVar('_write_2__A')  # <A>
    _write_3__A = typing.TypeVar('_write_3__A')  # <A>
    @typing.overload
    def write(self, byteBuffer: java.nio.ByteBuffer) -> java.util.concurrent.Future[int]: ...
    @typing.overload
    def write(self, byteBuffer: java.nio.ByteBuffer, long: int, timeUnit: java.util.concurrent.TimeUnit, a: _write_1__A, completionHandler: CompletionHandler[int, _write_1__A]) -> None: ...
    @typing.overload
    def write(self, byteBufferArray: typing.List[java.nio.ByteBuffer], int: int, int2: int, long: int, timeUnit: java.util.concurrent.TimeUnit, a: _write_2__A, completionHandler: CompletionHandler[int, _write_2__A]) -> None: ...
    @typing.overload
    def write(self, byteBuffer: java.nio.ByteBuffer, a: _write_3__A, completionHandler: CompletionHandler[int, _write_3__A]) -> None: ...

class DatagramChannel(java.nio.channels.spi.AbstractSelectableChannel, ByteChannel, ScatteringByteChannel, GatheringByteChannel, MulticastChannel):
    def bind(self, socketAddress: java.net.SocketAddress) -> 'DatagramChannel': ...
    def connect(self, socketAddress: java.net.SocketAddress) -> 'DatagramChannel': ...
    def disconnect(self) -> 'DatagramChannel': ...
    def getLocalAddress(self) -> java.net.SocketAddress: ...
    def getRemoteAddress(self) -> java.net.SocketAddress: ...
    def isConnected(self) -> bool: ...
    @typing.overload
    @staticmethod
    def open() -> 'DatagramChannel': ...
    @typing.overload
    @staticmethod
    def open(protocolFamily: java.net.ProtocolFamily) -> 'DatagramChannel': ...
    @typing.overload
    def read(self, byteBuffer: java.nio.ByteBuffer) -> int: ...
    @typing.overload
    def read(self, byteBufferArray: typing.List[java.nio.ByteBuffer], int: int, int2: int) -> int: ...
    @typing.overload
    def read(self, byteBufferArray: typing.List[java.nio.ByteBuffer]) -> int: ...
    def receive(self, byteBuffer: java.nio.ByteBuffer) -> java.net.SocketAddress: ...
    def send(self, byteBuffer: java.nio.ByteBuffer, socketAddress: java.net.SocketAddress) -> int: ...
    _setOption__T = typing.TypeVar('_setOption__T')  # <T>
    def setOption(self, socketOption: java.net.SocketOption[_setOption__T], t: _setOption__T) -> 'DatagramChannel': ...
    def socket(self) -> java.net.DatagramSocket: ...
    def validOps(self) -> int: ...
    @typing.overload
    def write(self, byteBuffer: java.nio.ByteBuffer) -> int: ...
    @typing.overload
    def write(self, byteBufferArray: typing.List[java.nio.ByteBuffer], int: int, int2: int) -> int: ...
    @typing.overload
    def write(self, byteBufferArray: typing.List[java.nio.ByteBuffer]) -> int: ...

class Pipe:
    @staticmethod
    def open() -> 'Pipe': ...
    def sink(self) -> 'Pipe.SinkChannel': ...
    def source(self) -> 'Pipe.SourceChannel': ...
    class SinkChannel(java.nio.channels.spi.AbstractSelectableChannel, WritableByteChannel, GatheringByteChannel):
        def validOps(self) -> int: ...
    class SourceChannel(java.nio.channels.spi.AbstractSelectableChannel, ReadableByteChannel, ScatteringByteChannel):
        def validOps(self) -> int: ...

class SeekableByteChannel(ByteChannel):
    @typing.overload
    def position(self, long: int) -> 'SeekableByteChannel': ...
    @typing.overload
    def position(self) -> int: ...
    def read(self, byteBuffer: java.nio.ByteBuffer) -> int: ...
    def size(self) -> int: ...
    def truncate(self, long: int) -> 'SeekableByteChannel': ...
    def write(self, byteBuffer: java.nio.ByteBuffer) -> int: ...

class SocketChannel(java.nio.channels.spi.AbstractSelectableChannel, ByteChannel, ScatteringByteChannel, GatheringByteChannel, NetworkChannel):
    def bind(self, socketAddress: java.net.SocketAddress) -> 'SocketChannel': ...
    def connect(self, socketAddress: java.net.SocketAddress) -> bool: ...
    def finishConnect(self) -> bool: ...
    def getLocalAddress(self) -> java.net.SocketAddress: ...
    def getRemoteAddress(self) -> java.net.SocketAddress: ...
    def isConnected(self) -> bool: ...
    def isConnectionPending(self) -> bool: ...
    @typing.overload
    @staticmethod
    def open() -> 'SocketChannel': ...
    @typing.overload
    @staticmethod
    def open(socketAddress: java.net.SocketAddress) -> 'SocketChannel': ...
    @typing.overload
    def read(self, byteBuffer: java.nio.ByteBuffer) -> int: ...
    @typing.overload
    def read(self, byteBufferArray: typing.List[java.nio.ByteBuffer], int: int, int2: int) -> int: ...
    @typing.overload
    def read(self, byteBufferArray: typing.List[java.nio.ByteBuffer]) -> int: ...
    _setOption__T = typing.TypeVar('_setOption__T')  # <T>
    def setOption(self, socketOption: java.net.SocketOption[_setOption__T], t: _setOption__T) -> 'SocketChannel': ...
    def shutdownInput(self) -> 'SocketChannel': ...
    def shutdownOutput(self) -> 'SocketChannel': ...
    def socket(self) -> java.net.Socket: ...
    def validOps(self) -> int: ...
    @typing.overload
    def write(self, byteBuffer: java.nio.ByteBuffer) -> int: ...
    @typing.overload
    def write(self, byteBufferArray: typing.List[java.nio.ByteBuffer], int: int, int2: int) -> int: ...
    @typing.overload
    def write(self, byteBufferArray: typing.List[java.nio.ByteBuffer]) -> int: ...

class FileChannel(java.nio.channels.spi.AbstractInterruptibleChannel, SeekableByteChannel, GatheringByteChannel, ScatteringByteChannel):
    def force(self, boolean: bool) -> None: ...
    @typing.overload
    def lock(self, long: int, long2: int, boolean: bool) -> FileLock: ...
    @typing.overload
    def lock(self) -> FileLock: ...
    def map(self, mapMode: 'FileChannel.MapMode', long: int, long2: int) -> java.nio.MappedByteBuffer: ...
    @typing.overload
    @staticmethod
    def open(path: typing.Union[java.nio.file.Path, jpype.protocol.SupportsPath], openOptionArray: typing.List[java.nio.file.OpenOption]) -> 'FileChannel': ...
    @typing.overload
    @staticmethod
    def open(path: typing.Union[java.nio.file.Path, jpype.protocol.SupportsPath], set: java.util.Set[java.nio.file.OpenOption], fileAttributeArray: typing.List[java.nio.file.attribute.FileAttribute[typing.Any]]) -> 'FileChannel': ...
    @typing.overload
    def position(self, long: int) -> 'FileChannel': ...
    @typing.overload
    def position(self) -> int: ...
    @typing.overload
    def read(self, byteBuffer: java.nio.ByteBuffer) -> int: ...
    @typing.overload
    def read(self, byteBuffer: java.nio.ByteBuffer, long: int) -> int: ...
    @typing.overload
    def read(self, byteBufferArray: typing.List[java.nio.ByteBuffer], int: int, int2: int) -> int: ...
    @typing.overload
    def read(self, byteBufferArray: typing.List[java.nio.ByteBuffer]) -> int: ...
    def size(self) -> int: ...
    def transferFrom(self, readableByteChannel: ReadableByteChannel, long: int, long2: int) -> int: ...
    def transferTo(self, long: int, long2: int, writableByteChannel: WritableByteChannel) -> int: ...
    def truncate(self, long: int) -> 'FileChannel': ...
    @typing.overload
    def tryLock(self, long: int, long2: int, boolean: bool) -> FileLock: ...
    @typing.overload
    def tryLock(self) -> FileLock: ...
    @typing.overload
    def write(self, byteBuffer: java.nio.ByteBuffer) -> int: ...
    @typing.overload
    def write(self, byteBuffer: java.nio.ByteBuffer, long: int) -> int: ...
    @typing.overload
    def write(self, byteBufferArray: typing.List[java.nio.ByteBuffer], int: int, int2: int) -> int: ...
    @typing.overload
    def write(self, byteBufferArray: typing.List[java.nio.ByteBuffer]) -> int: ...
    class MapMode:
        READ_ONLY: typing.ClassVar['FileChannel.MapMode'] = ...
        READ_WRITE: typing.ClassVar['FileChannel.MapMode'] = ...
        PRIVATE: typing.ClassVar['FileChannel.MapMode'] = ...
        def toString(self) -> str: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("java.nio.channels")``.

    AcceptPendingException: typing.Type[AcceptPendingException]
    AlreadyBoundException: typing.Type[AlreadyBoundException]
    AlreadyConnectedException: typing.Type[AlreadyConnectedException]
    AsynchronousByteChannel: typing.Type[AsynchronousByteChannel]
    AsynchronousChannel: typing.Type[AsynchronousChannel]
    AsynchronousChannelGroup: typing.Type[AsynchronousChannelGroup]
    AsynchronousCloseException: typing.Type[AsynchronousCloseException]
    AsynchronousFileChannel: typing.Type[AsynchronousFileChannel]
    AsynchronousServerSocketChannel: typing.Type[AsynchronousServerSocketChannel]
    AsynchronousSocketChannel: typing.Type[AsynchronousSocketChannel]
    ByteChannel: typing.Type[ByteChannel]
    CancelledKeyException: typing.Type[CancelledKeyException]
    Channel: typing.Type[Channel]
    Channels: typing.Type[Channels]
    ClosedByInterruptException: typing.Type[ClosedByInterruptException]
    ClosedChannelException: typing.Type[ClosedChannelException]
    ClosedSelectorException: typing.Type[ClosedSelectorException]
    CompletionHandler: typing.Type[CompletionHandler]
    ConnectionPendingException: typing.Type[ConnectionPendingException]
    DatagramChannel: typing.Type[DatagramChannel]
    FileChannel: typing.Type[FileChannel]
    FileLock: typing.Type[FileLock]
    FileLockInterruptionException: typing.Type[FileLockInterruptionException]
    GatheringByteChannel: typing.Type[GatheringByteChannel]
    IllegalBlockingModeException: typing.Type[IllegalBlockingModeException]
    IllegalChannelGroupException: typing.Type[IllegalChannelGroupException]
    IllegalSelectorException: typing.Type[IllegalSelectorException]
    InterruptedByTimeoutException: typing.Type[InterruptedByTimeoutException]
    InterruptibleChannel: typing.Type[InterruptibleChannel]
    MembershipKey: typing.Type[MembershipKey]
    MulticastChannel: typing.Type[MulticastChannel]
    NetworkChannel: typing.Type[NetworkChannel]
    NoConnectionPendingException: typing.Type[NoConnectionPendingException]
    NonReadableChannelException: typing.Type[NonReadableChannelException]
    NonWritableChannelException: typing.Type[NonWritableChannelException]
    NotYetBoundException: typing.Type[NotYetBoundException]
    NotYetConnectedException: typing.Type[NotYetConnectedException]
    OverlappingFileLockException: typing.Type[OverlappingFileLockException]
    Pipe: typing.Type[Pipe]
    ReadPendingException: typing.Type[ReadPendingException]
    ReadableByteChannel: typing.Type[ReadableByteChannel]
    ScatteringByteChannel: typing.Type[ScatteringByteChannel]
    SeekableByteChannel: typing.Type[SeekableByteChannel]
    SelectableChannel: typing.Type[SelectableChannel]
    SelectionKey: typing.Type[SelectionKey]
    Selector: typing.Type[Selector]
    ServerSocketChannel: typing.Type[ServerSocketChannel]
    ShutdownChannelGroupException: typing.Type[ShutdownChannelGroupException]
    SocketChannel: typing.Type[SocketChannel]
    UnresolvedAddressException: typing.Type[UnresolvedAddressException]
    UnsupportedAddressTypeException: typing.Type[UnsupportedAddressTypeException]
    WritableByteChannel: typing.Type[WritableByteChannel]
    WritePendingException: typing.Type[WritePendingException]
    spi: java.nio.channels.spi.__module_protocol__



import java.lang
import scala
import scala.collection
import scala.util
import scorex.crypto.authds
import scorex.crypto.authds.avltree.batch
import scorex.crypto.authds.legacy.treap
import scorex.crypto.hash
import scorex.util.encode
import typing



class AVLModifyProof(scorex.crypto.authds.TwoPartyProof, scala.Product, scala.Serializable):
    def __init__(self, key: typing.List[int], proofSeq: scala.collection.Seq[scorex.crypto.authds.AVLProofElement], hf: scorex.crypto.hash.CryptographicHash[typing.List[int]]): ...
    @staticmethod
    def apply(key: typing.List[int], proofSeq: scala.collection.Seq[scorex.crypto.authds.AVLProofElement], hf: scorex.crypto.hash.CryptographicHash[typing.List[int]]) -> 'AVLModifyProof': ...
    def bytes(self) -> typing.List[int]: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    @staticmethod
    def combineBytes(b1: int, b2: int) -> int: ...
    def copy(self, key: typing.List[int], proofSeq: scala.collection.Seq[scorex.crypto.authds.AVLProofElement], hf: scorex.crypto.hash.CryptographicHash[typing.List[int]]) -> 'AVLModifyProof': ...
    def copy$default$1(self) -> typing.List[int]: ...
    def copy$default$2(self) -> scala.collection.Seq[scorex.crypto.authds.AVLProofElement]: ...
    def dequeueBalance(self) -> int: ...
    def dequeueDirection(self) -> scorex.crypto.authds.Direction: ...
    def dequeueKey(self) -> typing.List[int]: ...
    def dequeueLeftLabel(self) -> typing.List[int]: ...
    def dequeueLevel(self) -> scorex.crypto.authds.legacy.treap.Level: ...
    def dequeueNextLeafKey(self) -> typing.List[int]: ...
    def dequeueRightLabel(self) -> typing.List[int]: ...
    def dequeueValue(self) -> typing.List[int]: ...
    @staticmethod
    def directionBalanceByte(dir: scorex.crypto.authds.ProofDirection, balance: scorex.crypto.authds.ProofBalance) -> int: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def initializeIterator(self) -> None: ...
    def key(self) -> typing.List[int]: ...
    @staticmethod
    def parseBytes(bytes: typing.List[int], keyLength: int, digestSize: int, hf: scorex.crypto.hash.CryptographicHash[typing.List[int]]) -> scala.util.Try['AVLModifyProof']: ...
    @staticmethod
    def parseBytes$default$2(bytes: typing.List[int]) -> int: ...
    @staticmethod
    def parseBytes$default$3(bytes: typing.List[int]) -> int: ...
    @staticmethod
    def parseBytes$default$4(bytes: typing.List[int]) -> scorex.crypto.hash.CryptographicHash[typing.List[int]]: ...
    @staticmethod
    def parseDirectionBalance(b: int) -> scala.Tuple2[scorex.crypto.authds.ProofDirection, scorex.crypto.authds.ProofBalance]: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def proofSeq(self) -> scala.collection.Seq[scorex.crypto.authds.AVLProofElement]: ...
    def scorex$crypto$authds$ProofIterator$$i(self) -> int: ...
    def scorex$crypto$authds$ProofIterator$$i_$eq(self, x$1: int) -> None: ...
    @staticmethod
    def splitBytes(b: int) -> scala.Tuple2[typing.Any, typing.Any]: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'AVLModifyProof') -> scala.Option[scala.Tuple2[typing.List[int], scala.collection.Seq[scorex.crypto.authds.AVLProofElement]]]: ...
    @typing.overload
    def verify(self, digest: typing.List[int], updateFn: scala.Function1[scala.Option[typing.List[int]], scala.util.Try[scala.Option[typing.List[int]]]]) -> scala.Option[typing.List[int]]: ...
    @typing.overload
    def verify(self, digest: typing.List[int], modification: scorex.crypto.authds.avltree.batch.Modification) -> scala.Option[typing.List[int]]: ...
    def verifyLookup(self, digest: typing.List[int], existence: bool) -> scala.Option[typing.List[int]]: ...

_AVLTree__HF = typing.TypeVar('_AVLTree__HF', bound=scorex.crypto.hash.CryptographicHash)  # <HF>
class AVLTree(scorex.crypto.authds.TwoPartyDictionary, typing.Generic[_AVLTree__HF]):
    def __init__(self, keyLength: int, valueLength: int, rootOpt: scala.Option['ProverNodes'], hf: _AVLTree__HF): ...
    _$lessinit$greater$default$2__HF = typing.TypeVar('_$lessinit$greater$default$2__HF', bound=scorex.crypto.hash.CryptographicHash)  # <HF>
    @staticmethod
    def $lessinit$greater$default$2() -> int: ...
    _$lessinit$greater$default$3__HF = typing.TypeVar('_$lessinit$greater$default$3__HF', bound=scorex.crypto.hash.CryptographicHash)  # <HF>
    @staticmethod
    def $lessinit$greater$default$3() -> scala.Option['ProverNodes']: ...
    _$lessinit$greater$default$4__HF = typing.TypeVar('_$lessinit$greater$default$4__HF', bound=scorex.crypto.hash.CryptographicHash)  # <HF>
    @staticmethod
    def $lessinit$greater$default$4(keyLength: int, valueLength: int, rootOpt: scala.Option['ProverNodes']) -> scorex.crypto.hash.Blake2b256.: ...
    def DefaultTopNode(self) -> 'Leaf': ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def rootHash(self) -> typing.List[int]: ...
    _run__O = typing.TypeVar('_run__O', bound=scorex.crypto.authds.avltree.batch.Operation)  # <O>
    def run(self, operation: _run__O) -> scala.util.Try[AVLModifyProof]: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...

class Node(scorex.crypto.authds.avltree.batch.ToStringHelper):
    @staticmethod
    def $init$($this: 'Node') -> None: ...
    def computeLabel(self) -> typing.List[int]: ...
    def label(self) -> typing.List[int]: ...
    def labelOpt(self) -> scala.Option[typing.List[int]]: ...
    def labelOpt_$eq(self, x$1: scala.Option[typing.List[int]]) -> None: ...

class InternalNode(Node):
    @staticmethod
    def $init$($this: 'InternalNode') -> None: ...
    def _balance(self) -> int: ...
    def _balance_$eq(self, x$1: int) -> None: ...
    def balance(self) -> int: ...
    def balance_$eq(self, balance: int) -> None: ...
    def computeLabel(self) -> typing.List[int]: ...
    def hf(self) -> scorex.crypto.hash.CryptographicHash[typing.List[int]]: ...
    def leftLabel(self) -> typing.List[int]: ...
    def rightLabel(self) -> typing.List[int]: ...

class LabelOnlyNode(Node, scala.Product, scala.Serializable):
    def __init__(self, l: typing.List[int]): ...
    _andThen__A = typing.TypeVar('_andThen__A')  # <A>
    @staticmethod
    def andThen(g: scala.Function1['LabelOnlyNode', _andThen__A]) -> scala.Function1[typing.List[int], _andThen__A]: ...
    @staticmethod
    def apply(l: typing.List[int]) -> 'LabelOnlyNode': ...
    def arrayToString(self, a: typing.List[int]) -> java.lang.String: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _compose__A = typing.TypeVar('_compose__A')  # <A>
    @staticmethod
    def compose(g: scala.Function1[_compose__A, typing.List[int]]) -> scala.Function1[_compose__A, 'LabelOnlyNode']: ...
    def computeLabel(self) -> typing.List[int]: ...
    def copy(self, l: typing.List[int]) -> 'LabelOnlyNode': ...
    def copy$default$1(self) -> typing.List[int]: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def l(self) -> typing.List[int]: ...
    def label(self) -> typing.List[int]: ...
    def labelOpt(self) -> scala.Option[typing.List[int]]: ...
    def labelOpt_$eq(self, x$1: scala.Option[typing.List[int]]) -> None: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'LabelOnlyNode') -> scala.Option[typing.List[int]]: ...

class ProverNodes(Node):
    @staticmethod
    def $init$($this: 'ProverNodes') -> None: ...
    def height(self) -> int: ...
    def height_$eq(self, x$1: int) -> None: ...
    def isNew(self) -> bool: ...
    def isNew_$eq(self, x$1: bool) -> None: ...
    def key(self) -> typing.List[int]: ...
    def visited(self) -> bool: ...
    def visited_$eq(self, x$1: bool) -> None: ...

class VerifierNodes(Node): ...

class Leaf(ProverNodes, VerifierNodes, scala.Product, scala.Serializable):
    def __init__(self, key: typing.List[int], _value: typing.List[int], _nextLeafKey: typing.List[int], hf: scorex.crypto.hash.CryptographicHash[typing.List[int]]): ...
    def _nextLeafKey$access$2(self) -> typing.List[int]: ...
    def _value$access$1(self) -> typing.List[int]: ...
    @staticmethod
    def apply(key: typing.List[int], _value: typing.List[int], _nextLeafKey: typing.List[int], hf: scorex.crypto.hash.CryptographicHash[typing.List[int]]) -> 'Leaf': ...
    def arrayToString(self, a: typing.List[int]) -> java.lang.String: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    def computeLabel(self) -> typing.List[int]: ...
    def copy(self, key: typing.List[int], _value: typing.List[int], _nextLeafKey: typing.List[int], hf: scorex.crypto.hash.CryptographicHash[typing.List[int]]) -> 'Leaf': ...
    def copy$default$1(self) -> typing.List[int]: ...
    def copy$default$2(self) -> typing.List[int]: ...
    def copy$default$3(self) -> typing.List[int]: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def height(self) -> int: ...
    def height_$eq(self, x$1: int) -> None: ...
    def hf(self) -> scorex.crypto.hash.CryptographicHash[typing.List[int]]: ...
    def isNew(self) -> bool: ...
    def isNew_$eq(self, x$1: bool) -> None: ...
    def key(self) -> typing.List[int]: ...
    def label(self) -> typing.List[int]: ...
    def labelOpt(self) -> scala.Option[typing.List[int]]: ...
    def labelOpt_$eq(self, x$1: scala.Option[typing.List[int]]) -> None: ...
    def nextLeafKey(self) -> typing.List[int]: ...
    def nextLeafKey_$eq(self, newNextLeafKey: typing.List[int]) -> None: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def scorex$crypto$authds$legacy$avltree$Leaf$$_nextLeafKey(self) -> typing.List[int]: ...
    def scorex$crypto$authds$legacy$avltree$Leaf$$_value(self) -> typing.List[int]: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'Leaf') -> scala.Option[scala.Tuple3[typing.List[int], typing.List[int], typing.List[int]]]: ...
    def value(self) -> typing.List[int]: ...
    def value_$eq(self, newValue: typing.List[int]) -> None: ...
    def visited(self) -> bool: ...
    def visited_$eq(self, x$1: bool) -> None: ...

class ProverNode(ProverNodes, InternalNode, scala.Product, scala.Serializable):
    def __init__(self, key: typing.List[int], _left: ProverNodes, _right: ProverNodes, _balance: int, hf: scorex.crypto.hash.CryptographicHash[typing.List[int]]): ...
    @staticmethod
    def $lessinit$greater$default$4() -> int: ...
    def _balance(self) -> int: ...
    def _balance$access$3(self) -> int: ...
    def _balance_$eq(self, x$1: int) -> None: ...
    def _left$access$1(self) -> ProverNodes: ...
    def _right$access$2(self) -> ProverNodes: ...
    @staticmethod
    def apply(key: typing.List[int], _left: ProverNodes, _right: ProverNodes, _balance: int, hf: scorex.crypto.hash.CryptographicHash[typing.List[int]]) -> 'ProverNode': ...
    @staticmethod
    def apply$default$4() -> int: ...
    def arrayToString(self, a: typing.List[int]) -> java.lang.String: ...
    def balance(self) -> int: ...
    def balance_$eq(self, balance: int) -> None: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    def checkHeight(self) -> bool: ...
    def computeLabel(self) -> typing.List[int]: ...
    def copy(self, key: typing.List[int], _left: ProverNodes, _right: ProverNodes, _balance: int, hf: scorex.crypto.hash.CryptographicHash[typing.List[int]]) -> 'ProverNode': ...
    def copy$default$1(self) -> typing.List[int]: ...
    def copy$default$2(self) -> ProverNodes: ...
    def copy$default$3(self) -> ProverNodes: ...
    def copy$default$4(self) -> int: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def height(self) -> int: ...
    def height_$eq(self, x$1: int) -> None: ...
    def hf(self) -> scorex.crypto.hash.CryptographicHash[typing.List[int]]: ...
    def isNew(self) -> bool: ...
    def isNew_$eq(self, x$1: bool) -> None: ...
    def key(self) -> typing.List[int]: ...
    def label(self) -> typing.List[int]: ...
    def labelOpt(self) -> scala.Option[typing.List[int]]: ...
    def labelOpt_$eq(self, x$1: scala.Option[typing.List[int]]) -> None: ...
    def left(self) -> ProverNodes: ...
    def leftLabel(self) -> typing.List[int]: ...
    def left_$eq(self, newLeft: ProverNodes) -> None: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def right(self) -> ProverNodes: ...
    def rightLabel(self) -> typing.List[int]: ...
    def right_$eq(self, newRight: ProverNodes) -> None: ...
    def scorex$crypto$authds$legacy$avltree$ProverNode$$_left(self) -> ProverNodes: ...
    def scorex$crypto$authds$legacy$avltree$ProverNode$$_right(self) -> ProverNodes: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'ProverNode') -> scala.Option[scala.Tuple4[typing.List[int], ProverNodes, ProverNodes, typing.Any]]: ...
    def visited(self) -> bool: ...
    def visited_$eq(self, x$1: bool) -> None: ...

class VerifierNode(VerifierNodes, InternalNode, scala.Product, scala.Serializable):
    def __init__(self, _left: Node, _right: Node, _balance: int, hf: scorex.crypto.hash.CryptographicHash[typing.List[int]]): ...
    def _balance(self) -> int: ...
    def _balance$access$2(self) -> int: ...
    def _balance_$eq(self, x$1: int) -> None: ...
    def _left$access$0(self) -> Node: ...
    def _right$access$1(self) -> Node: ...
    @staticmethod
    def apply(_left: Node, _right: Node, _balance: int, hf: scorex.crypto.hash.CryptographicHash[typing.List[int]]) -> 'VerifierNode': ...
    def arrayToString(self, a: typing.List[int]) -> java.lang.String: ...
    def balance(self) -> int: ...
    def balance_$eq(self, balance: int) -> None: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    def computeLabel(self) -> typing.List[int]: ...
    def copy(self, _left: Node, _right: Node, _balance: int, hf: scorex.crypto.hash.CryptographicHash[typing.List[int]]) -> 'VerifierNode': ...
    def copy$default$1(self) -> Node: ...
    def copy$default$2(self) -> Node: ...
    def copy$default$3(self) -> int: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def hf(self) -> scorex.crypto.hash.CryptographicHash[typing.List[int]]: ...
    def label(self) -> typing.List[int]: ...
    def labelOpt(self) -> scala.Option[typing.List[int]]: ...
    def labelOpt_$eq(self, x$1: scala.Option[typing.List[int]]) -> None: ...
    def left(self) -> Node: ...
    def leftLabel(self) -> typing.List[int]: ...
    def left_$eq(self, newLeft: Node) -> None: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def right(self) -> Node: ...
    def rightLabel(self) -> typing.List[int]: ...
    def right_$eq(self, newRight: Node) -> None: ...
    def scorex$crypto$authds$legacy$avltree$VerifierNode$$_left(self) -> Node: ...
    def scorex$crypto$authds$legacy$avltree$VerifierNode$$_right(self) -> Node: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'VerifierNode') -> scala.Option[scala.Tuple3[Node, Node, typing.Any]]: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("scorex.crypto.authds.legacy.avltree")``.

    AVLModifyProof: typing.Type[AVLModifyProof]
    AVLTree: typing.Type[AVLTree]
    InternalNode: typing.Type[InternalNode]
    LabelOnlyNode: typing.Type[LabelOnlyNode]
    Leaf: typing.Type[Leaf]
    Node: typing.Type[Node]
    ProverNode: typing.Type[ProverNode]
    ProverNodes: typing.Type[ProverNodes]
    VerifierNode: typing.Type[VerifierNode]
    VerifierNodes: typing.Type[VerifierNodes]



import java.lang
import scala
import scala.collection
import scala.collection.immutable
import scala.collection.mutable
import scala.math
import scala.runtime
import scala.util
import scorex.crypto.hash
import typing



class BlockchainSimulator:
    @staticmethod
    def PubKeyLength() -> int: ...
    @staticmethod
    def currentGodBalance() -> int: ...
    @staticmethod
    def currentGodBalance_$eq(x$1: int) -> None: ...
    @staticmethod
    def delayedInit(body: scala.Function0[scala.runtime.BoxedUnit]) -> None: ...
    @staticmethod
    def emptyState() -> 'SparseMerkleTree'[typing.List[int]]: ...
    @staticmethod
    def executionStart() -> int: ...
    @staticmethod
    def godAccount() -> typing.List[int]: ...
    @staticmethod
    def godBalance() -> int: ...
    @staticmethod
    def godProof() -> 'SparseMerkleProof'[typing.List[int]]: ...
    @staticmethod
    def godProof_$eq(x$1: 'SparseMerkleProof'[typing.List[int]]) -> None: ...
    @staticmethod
    def godProofs() -> scala.collection.Seq['SparseMerkleProof'[typing.List[int]]]: ...
    @staticmethod
    def height() -> int: ...
    @staticmethod
    def hf() -> scorex.crypto.hash.CryptographicHash[typing.List[int]]: ...
    @staticmethod
    def initialState() -> 'SparseMerkleTree'[typing.List[int]]: ...
    @staticmethod
    def main(args: typing.List[java.lang.String]) -> None: ...
    @staticmethod
    def maxTxsCacheSize() -> int: ...
    @staticmethod
    def numOfBlocks() -> int: ...
    @staticmethod
    def txAmount() -> int: ...
    @staticmethod
    def txsCache() -> scala.collection.mutable.ArrayBuffer[scala.runtime.Nothing.]: ...
    @staticmethod
    def txsPerBlock() -> int: ...
    class Block(scala.Product, scala.Serializable):
        def __init__(self, transactions: scala.collection.Seq['BlockchainSimulator.Transaction']): ...
        def canEqual(self, x$1: typing.Any) -> bool: ...
        def copy(self, transactions: scala.collection.Seq['BlockchainSimulator.Transaction']) -> 'BlockchainSimulator.Block': ...
        def copy$default$1(self) -> scala.collection.Seq['BlockchainSimulator.Transaction']: ...
        def equals(self, x$1: typing.Any) -> bool: ...
        def hashCode(self) -> int: ...
        def productArity(self) -> int: ...
        def productElement(self, x$1: int) -> typing.Any: ...
        def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
        def productPrefix(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
        def transactions(self) -> scala.collection.Seq['BlockchainSimulator.Transaction']: ...
    class Block$(scala.runtime.AbstractFunction1[scala.collection.Seq['BlockchainSimulator.Transaction'], 'BlockchainSimulator.Block'], scala.Serializable):
        MODULE$: typing.ClassVar['BlockchainSimulator.Block.'] = ...
        def __init__(self): ...
        def apply(self, transactions: scala.collection.Seq['BlockchainSimulator.Transaction']) -> 'BlockchainSimulator.Block': ...
        def toString(self) -> java.lang.String: ...
        def unapply(self, x$0: 'BlockchainSimulator.Block') -> scala.Option[scala.collection.Seq['BlockchainSimulator.Transaction']]: ...
    class Transaction(scala.Product, scala.Serializable):
        def __init__(self, amount: int, sender: typing.List[int], recipient: typing.List[int], coinBalance: int, coinProof: 'SparseMerkleProof'[typing.List[int]]): ...
        def amount(self) -> int: ...
        def canEqual(self, x$1: typing.Any) -> bool: ...
        def coinBalance(self) -> int: ...
        def coinProof(self) -> 'SparseMerkleProof'[typing.List[int]]: ...
        def copy(self, amount: int, sender: typing.List[int], recipient: typing.List[int], coinBalance: int, coinProof: 'SparseMerkleProof'[typing.List[int]]) -> 'BlockchainSimulator.Transaction': ...
        def copy$default$1(self) -> int: ...
        def copy$default$2(self) -> typing.List[int]: ...
        def copy$default$3(self) -> typing.List[int]: ...
        def copy$default$4(self) -> int: ...
        def copy$default$5(self) -> 'SparseMerkleProof'[typing.List[int]]: ...
        def equals(self, x$1: typing.Any) -> bool: ...
        def hashCode(self) -> int: ...
        def productArity(self) -> int: ...
        def productElement(self, x$1: int) -> typing.Any: ...
        def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
        def productPrefix(self) -> java.lang.String: ...
        def recipient(self) -> typing.List[int]: ...
        def sender(self) -> typing.List[int]: ...
        def toString(self) -> java.lang.String: ...
    class Transaction$(scala.Serializable):
        MODULE$: typing.ClassVar['BlockchainSimulator.Transaction.'] = ...
        def __init__(self): ...
        def apply(self, amount: int, sender: typing.List[int], recipient: typing.List[int], coinBalance: int, coinProof: 'SparseMerkleProof'[typing.List[int]]) -> 'BlockchainSimulator.Transaction': ...
        def coinBytes(self, pubKey: typing.List[int], balance: int) -> scala.Some[typing.List[int]]: ...
        def process(self, tx: 'BlockchainSimulator.Transaction', state: 'SparseMerkleTree'[typing.List[int]]) -> scala.util.Try[scala.Tuple2['SparseMerkleTree'[typing.List[int]], scala.collection.Seq['SparseMerkleProof'[typing.List[int]]]]]: ...
        def unapply(self, x$0: 'BlockchainSimulator.Transaction') -> scala.Option[scala.Tuple5[typing.Any, typing.List[int], typing.List[int], typing.Any, 'SparseMerkleProof'[typing.List[int]]]]: ...

_Node__D = typing.TypeVar('_Node__D', bound=typing.List[int])  # <D>
class Node(typing.Generic[_Node__D]):
    def hash(self) -> _Node__D: ...
    def isNull(self) -> bool: ...

class OpsBenchmark:
    @staticmethod
    def bSize() -> int: ...
    @staticmethod
    def delayedInit(body: scala.Function0[scala.runtime.BoxedUnit]) -> None: ...
    @staticmethod
    def executionStart() -> int: ...
    @staticmethod
    def height() -> int: ...
    @staticmethod
    def hf() -> scorex.crypto.hash.CryptographicHash[typing.List[int]]: ...
    @staticmethod
    def main(args: typing.List[java.lang.String]) -> None: ...
    @staticmethod
    def proof() -> 'SparseMerkleProof'[typing.List[int]]: ...
    @staticmethod
    def proof_$eq(x$1: 'SparseMerkleProof'[typing.List[int]]) -> None: ...
    @staticmethod
    def tb() -> int: ...
    @staticmethod
    def tb0() -> int: ...
    @staticmethod
    def te() -> int: ...
    @staticmethod
    def te0() -> int: ...
    @staticmethod
    def tp() -> int: ...
    @staticmethod
    def tp0() -> int: ...
    @staticmethod
    def tree() -> 'SparseMerkleTree'[typing.List[int]]: ...
    @staticmethod
    def tree_$eq(x$1: 'SparseMerkleTree'[typing.List[int]]) -> None: ...
    @staticmethod
    def tu() -> int: ...
    @staticmethod
    def tv() -> int: ...
    @staticmethod
    def tv0() -> int: ...

_SparseMerkleProof__D = typing.TypeVar('_SparseMerkleProof__D', bound=typing.List[int])  # <D>
class SparseMerkleProof(scala.Product, scala.Serializable, typing.Generic[_SparseMerkleProof__D]):
    def __init__(self, idx: scala.math.BigInt, leafDataOpt: scala.Option[typing.List[int]], levels: scala.collection.immutable.Vector[scala.Option[_SparseMerkleProof__D]]): ...
    _apply__D = typing.TypeVar('_apply__D', bound=typing.List[int])  # <D>
    @staticmethod
    def apply(idx: scala.math.BigInt, leafDataOpt: scala.Option[typing.List[int]], levels: scala.collection.immutable.Vector[scala.Option[_apply__D]]) -> 'SparseMerkleProof'[_apply__D]: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _copy__D = typing.TypeVar('_copy__D', bound=typing.List[int])  # <D>
    def copy(self, idx: scala.math.BigInt, leafDataOpt: scala.Option[typing.List[int]], levels: scala.collection.immutable.Vector[scala.Option[typing.List[int]]]) -> 'SparseMerkleProof'[typing.List[int]]: ...
    _copy$default$1__D = typing.TypeVar('_copy$default$1__D', bound=typing.List[int])  # <D>
    def copy$default$1(self) -> scala.math.BigInt: ...
    _copy$default$2__D = typing.TypeVar('_copy$default$2__D', bound=typing.List[int])  # <D>
    def copy$default$2(self) -> scala.Option[typing.List[int]]: ...
    _copy$default$3__D = typing.TypeVar('_copy$default$3__D', bound=typing.List[int])  # <D>
    def copy$default$3(self) -> scala.collection.immutable.Vector[scala.Option[typing.List[int]]]: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def idx(self) -> scala.math.BigInt: ...
    def leafDataOpt(self) -> scala.Option[typing.List[int]]: ...
    def levels(self) -> scala.collection.immutable.Vector[scala.Option[_SparseMerkleProof__D]]: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def propagateChanges(self, leafDataOpt: scala.Option[typing.List[int]], hf: scorex.crypto.hash.CryptographicHash[_SparseMerkleProof__D]) -> scala.Tuple2[scala.Option[_SparseMerkleProof__D], scala.collection.immutable.Vector[scala.Option[_SparseMerkleProof__D]]]: ...
    def toString(self) -> java.lang.String: ...
    _unapply__D = typing.TypeVar('_unapply__D', bound=typing.List[int])  # <D>
    @staticmethod
    def unapply(x$0: 'SparseMerkleProof'[_unapply__D]) -> scala.Option[scala.Tuple3[scala.math.BigInt, scala.Option[typing.List[int]], scala.collection.immutable.Vector[scala.Option[_unapply__D]]]]: ...
    @typing.overload
    def valid(self, expectedRootHash: scala.Option[_SparseMerkleProof__D], height: int, hf: scorex.crypto.hash.CryptographicHash[_SparseMerkleProof__D]) -> bool: ...
    @typing.overload
    def valid(self, tree: 'SparseMerkleTree'[_SparseMerkleProof__D], hf: scorex.crypto.hash.CryptographicHash[_SparseMerkleProof__D]) -> bool: ...

_SparseMerkleTree__D = typing.TypeVar('_SparseMerkleTree__D', bound=typing.List[int])  # <D>
class SparseMerkleTree(typing.Generic[_SparseMerkleTree__D]):
    def __init__(self, rootDigest: scala.Option[_SparseMerkleTree__D], height: int, lastProof: SparseMerkleProof[_SparseMerkleTree__D], hf: scorex.crypto.hash.CryptographicHash[_SparseMerkleTree__D]): ...
    _emptyTree__D = typing.TypeVar('_emptyTree__D', bound=typing.List[int])  # <D>
    @staticmethod
    def emptyTree(height: int, hf: scorex.crypto.hash.CryptographicHash[_emptyTree__D]) -> 'SparseMerkleTree'[_emptyTree__D]: ...
    def height(self) -> int: ...
    @staticmethod
    def indexToBits(idx: scala.math.BigInt, height: int) -> scala.collection.immutable.IndexedSeq[typing.Any]: ...
    @staticmethod
    def indexToBitsReverse(idx: scala.math.BigInt, height: int) -> scala.collection.immutable.IndexedSeq[typing.Any]: ...
    def lastIndex(self) -> scala.math.BigInt: ...
    def lastProof(self) -> SparseMerkleProof[_SparseMerkleTree__D]: ...
    @staticmethod
    def passAllFilterFn() -> scala.Function2[scala.math.BigInt, scala.Option[typing.List[int]], typing.Any]: ...
    def rootDigest(self) -> scala.Option[_SparseMerkleTree__D]: ...
    def update(self, proof: SparseMerkleProof[_SparseMerkleTree__D], newLeafData: scala.Option[typing.List[int]], proofsToUpdate: scala.collection.Seq[SparseMerkleProof[_SparseMerkleTree__D]], filterFn: scala.Function2[scala.math.BigInt, scala.Option[typing.List[int]], typing.Any]) -> scala.util.Try[scala.Tuple2['SparseMerkleTree'[_SparseMerkleTree__D], scala.collection.Seq[SparseMerkleProof[_SparseMerkleTree__D]]]]: ...
    def update$default$3(self) -> scala.collection.Seq[SparseMerkleProof[_SparseMerkleTree__D]]: ...
    def update$default$4(self) -> scala.Function2[scala.math.BigInt, scala.Option[typing.List[int]], typing.Any]: ...
    _zeroProof__D = typing.TypeVar('_zeroProof__D', bound=typing.List[int])  # <D>
    @staticmethod
    def zeroProof(height: int) -> SparseMerkleProof[_zeroProof__D]: ...

_InternalNode__D = typing.TypeVar('_InternalNode__D', bound=typing.List[int])  # <D>
class InternalNode(Node[_InternalNode__D], scala.Product, scala.Serializable, typing.Generic[_InternalNode__D]):
    def __init__(self, left: scala.Option[Node[_InternalNode__D]], right: scala.Option[Node[_InternalNode__D]], hf: scorex.crypto.hash.CryptographicHash[_InternalNode__D]): ...
    _apply__D = typing.TypeVar('_apply__D', bound=typing.List[int])  # <D>
    @staticmethod
    def apply(left: scala.Option[Node[_apply__D]], right: scala.Option[Node[_apply__D]], hf: scorex.crypto.hash.CryptographicHash[_apply__D]) -> 'InternalNode'[_apply__D]: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _copy__D = typing.TypeVar('_copy__D', bound=typing.List[int])  # <D>
    def copy(self, left: scala.Option[Node[typing.List[int]]], right: scala.Option[Node[typing.List[int]]], hf: scorex.crypto.hash.CryptographicHash[typing.List[int]]) -> 'InternalNode'[typing.List[int]]: ...
    _copy$default$1__D = typing.TypeVar('_copy$default$1__D', bound=typing.List[int])  # <D>
    def copy$default$1(self) -> scala.Option[Node[typing.List[int]]]: ...
    _copy$default$2__D = typing.TypeVar('_copy$default$2__D', bound=typing.List[int])  # <D>
    def copy$default$2(self) -> scala.Option[Node[typing.List[int]]]: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hash(self) -> _InternalNode__D: ...
    def hashCode(self) -> int: ...
    def hf(self) -> scorex.crypto.hash.CryptographicHash[_InternalNode__D]: ...
    def isNull(self) -> bool: ...
    def left(self) -> scala.Option[Node[_InternalNode__D]]: ...
    def leftHash(self) -> typing.List[int]: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def right(self) -> scala.Option[Node[_InternalNode__D]]: ...
    def rightHash(self) -> typing.List[int]: ...
    def toString(self) -> java.lang.String: ...
    _unapply__D = typing.TypeVar('_unapply__D', bound=typing.List[int])  # <D>
    @staticmethod
    def unapply(x$0: 'InternalNode'[_unapply__D]) -> scala.Option[scala.Tuple2[scala.Option[Node[_unapply__D]], scala.Option[Node[_unapply__D]]]]: ...

_Leaf__D = typing.TypeVar('_Leaf__D', bound=typing.List[int])  # <D>
class Leaf(Node[_Leaf__D], scala.Product, scala.Serializable, typing.Generic[_Leaf__D]):
    def __init__(self, idx: scala.math.BigInt, data: typing.List[int], hf: scorex.crypto.hash.CryptographicHash[_Leaf__D]): ...
    _apply__D = typing.TypeVar('_apply__D', bound=typing.List[int])  # <D>
    @staticmethod
    def apply(idx: scala.math.BigInt, data: typing.List[int], hf: scorex.crypto.hash.CryptographicHash[_apply__D]) -> 'Leaf'[_apply__D]: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _copy__D = typing.TypeVar('_copy__D', bound=typing.List[int])  # <D>
    def copy(self, idx: scala.math.BigInt, data: typing.List[int], hf: scorex.crypto.hash.CryptographicHash[typing.List[int]]) -> 'Leaf'[typing.List[int]]: ...
    _copy$default$1__D = typing.TypeVar('_copy$default$1__D', bound=typing.List[int])  # <D>
    def copy$default$1(self) -> scala.math.BigInt: ...
    _copy$default$2__D = typing.TypeVar('_copy$default$2__D', bound=typing.List[int])  # <D>
    def copy$default$2(self) -> typing.List[int]: ...
    def data(self) -> typing.List[int]: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hash(self) -> _Leaf__D: ...
    def hashCode(self) -> int: ...
    def hf(self) -> scorex.crypto.hash.CryptographicHash[_Leaf__D]: ...
    def idx(self) -> scala.math.BigInt: ...
    def isNull(self) -> bool: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def toString(self) -> java.lang.String: ...
    _unapply__D = typing.TypeVar('_unapply__D', bound=typing.List[int])  # <D>
    @staticmethod
    def unapply(x$0: 'Leaf'[_unapply__D]) -> scala.Option[scala.Tuple2[scala.math.BigInt, typing.List[int]]]: ...

_LeafHash__D = typing.TypeVar('_LeafHash__D', bound=typing.List[int])  # <D>
class LeafHash(Node[_LeafHash__D], scala.Product, scala.Serializable, typing.Generic[_LeafHash__D]):
    def __init__(self, hash: _LeafHash__D): ...
    _apply__D = typing.TypeVar('_apply__D', bound=typing.List[int])  # <D>
    @staticmethod
    def apply(hash: _apply__D) -> 'LeafHash'[_apply__D]: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _copy__D = typing.TypeVar('_copy__D', bound=typing.List[int])  # <D>
    def copy(self, hash: typing.List[int]) -> 'LeafHash'[typing.List[int]]: ...
    _copy$default$1__D = typing.TypeVar('_copy$default$1__D', bound=typing.List[int])  # <D>
    def copy$default$1(self) -> typing.List[int]: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hash(self) -> _LeafHash__D: ...
    def hashCode(self) -> int: ...
    def isNull(self) -> bool: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def toString(self) -> java.lang.String: ...
    _unapply__D = typing.TypeVar('_unapply__D', bound=typing.List[int])  # <D>
    @staticmethod
    def unapply(x$0: 'LeafHash'[_unapply__D]) -> scala.Option[_unapply__D]: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("scorex.crypto.authds.merkle.sparse")``.

    BlockchainSimulator: typing.Type[BlockchainSimulator]
    InternalNode: typing.Type[InternalNode]
    Leaf: typing.Type[Leaf]
    LeafHash: typing.Type[LeafHash]
    Node: typing.Type[Node]
    OpsBenchmark: typing.Type[OpsBenchmark]
    SparseMerkleProof: typing.Type[SparseMerkleProof]
    SparseMerkleTree: typing.Type[SparseMerkleTree]



import java.lang
import scala
import scala.collection
import scala.util
import scorex.crypto.hash
import scorex.util
import sigmastate
import special.collection
import typing



class ErgoAlgos(scorex.util.ScorexEncoding):
    @staticmethod
    def $init$($this: 'ErgoAlgos') -> None: ...
    def decode(self, str: typing.Union[java.lang.String, str]) -> scala.util.Try[typing.List[int]]: ...
    def decodeUnsafe(self, str: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @typing.overload
    def encode(self, bytes: typing.List[int]) -> java.lang.String: ...
    @typing.overload
    def encode(self, bytes: special.collection.Coll[typing.Any]) -> java.lang.String: ...
    def hash(self) -> scorex.crypto.hash.Blake2b256.: ...
    def org$ergoplatform$settings$ErgoAlgos$_setter_$hash_$eq(self, x$1: scorex.crypto.hash.Blake2b256.) -> None: ...

class MonetarySettings(scala.Product, scala.Serializable):
    def __init__(self, fixedRatePeriod: int, epochLength: int, fixedRate: int, oneEpochReduction: int, minerRewardDelay: int, foundersInitialReward: int): ...
    @staticmethod
    def $lessinit$greater$default$1() -> int: ...
    @staticmethod
    def $lessinit$greater$default$2() -> int: ...
    @staticmethod
    def $lessinit$greater$default$3() -> int: ...
    @staticmethod
    def $lessinit$greater$default$4() -> int: ...
    @staticmethod
    def $lessinit$greater$default$5() -> int: ...
    @staticmethod
    def $lessinit$greater$default$6() -> int: ...
    @staticmethod
    def apply(fixedRatePeriod: int, epochLength: int, fixedRate: int, oneEpochReduction: int, minerRewardDelay: int, foundersInitialReward: int) -> 'MonetarySettings': ...
    @staticmethod
    def apply$default$1() -> int: ...
    @staticmethod
    def apply$default$2() -> int: ...
    @staticmethod
    def apply$default$3() -> int: ...
    @staticmethod
    def apply$default$4() -> int: ...
    @staticmethod
    def apply$default$5() -> int: ...
    @staticmethod
    def apply$default$6() -> int: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    def copy(self, fixedRatePeriod: int, epochLength: int, fixedRate: int, oneEpochReduction: int, minerRewardDelay: int, foundersInitialReward: int) -> 'MonetarySettings': ...
    def copy$default$1(self) -> int: ...
    def copy$default$2(self) -> int: ...
    def copy$default$3(self) -> int: ...
    def copy$default$4(self) -> int: ...
    def copy$default$5(self) -> int: ...
    def copy$default$6(self) -> int: ...
    @staticmethod
    def curried() -> scala.Function1[typing.Any, scala.Function1[typing.Any, scala.Function1[typing.Any, scala.Function1[typing.Any, scala.Function1[typing.Any, scala.Function1[typing.Any, 'MonetarySettings']]]]]]: ...
    def emissionBoxProposition(self) -> sigmastate.Values.ErgoTree: ...
    def epochLength(self) -> int: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def feeProposition(self) -> sigmastate.Values.ErgoTree: ...
    def feePropositionBytes(self) -> typing.List[int]: ...
    def fixedRate(self) -> int: ...
    def fixedRatePeriod(self) -> int: ...
    def foundersBoxProposition(self) -> sigmastate.Values.ErgoTree: ...
    def foundersInitialReward(self) -> int: ...
    def hashCode(self) -> int: ...
    def minerRewardDelay(self) -> int: ...
    def oneEpochReduction(self) -> int: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def tupled() -> scala.Function1[scala.Tuple6[typing.Any, typing.Any, typing.Any, typing.Any, typing.Any, typing.Any], 'MonetarySettings']: ...
    @staticmethod
    def unapply(x$0: 'MonetarySettings') -> scala.Option[scala.Tuple6[typing.Any, typing.Any, typing.Any, typing.Any, typing.Any, typing.Any]]: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("org.ergoplatform.settings")``.

    ErgoAlgos: typing.Type[ErgoAlgos]
    MonetarySettings: typing.Type[MonetarySettings]



import java.io
import java.util
import typing



class LocaleServiceProvider:
    def getAvailableLocales(self) -> typing.List[java.util.Locale]: ...
    def isSupportedLocale(self, locale: java.util.Locale) -> bool: ...

class ResourceBundleControlProvider:
    def getControl(self, string: str) -> java.util.ResourceBundle.Control: ...

class ResourceBundleProvider:
    def getBundle(self, string: str, locale: java.util.Locale) -> java.util.ResourceBundle: ...

class ToolProvider:
    @staticmethod
    def findFirst(string: str) -> java.util.Optional['ToolProvider']: ...
    def name(self) -> str: ...
    @typing.overload
    def run(self, printWriter: java.io.PrintWriter, printWriter2: java.io.PrintWriter, stringArray: typing.List[str]) -> int: ...
    @typing.overload
    def run(self, printStream: java.io.PrintStream, printStream2: java.io.PrintStream, stringArray: typing.List[str]) -> int: ...

class AbstractResourceBundleProvider(ResourceBundleProvider):
    def getBundle(self, string: str, locale: java.util.Locale) -> java.util.ResourceBundle: ...

class CalendarDataProvider(LocaleServiceProvider):
    def getFirstDayOfWeek(self, locale: java.util.Locale) -> int: ...
    def getMinimalDaysInFirstWeek(self, locale: java.util.Locale) -> int: ...

class CalendarNameProvider(LocaleServiceProvider):
    def getDisplayName(self, string: str, int: int, int2: int, int3: int, locale: java.util.Locale) -> str: ...
    def getDisplayNames(self, string: str, int: int, int2: int, locale: java.util.Locale) -> java.util.Map[str, int]: ...

class CurrencyNameProvider(LocaleServiceProvider):
    def getDisplayName(self, string: str, locale: java.util.Locale) -> str: ...
    def getSymbol(self, string: str, locale: java.util.Locale) -> str: ...

class LocaleNameProvider(LocaleServiceProvider):
    def getDisplayCountry(self, string: str, locale: java.util.Locale) -> str: ...
    def getDisplayLanguage(self, string: str, locale: java.util.Locale) -> str: ...
    def getDisplayScript(self, string: str, locale: java.util.Locale) -> str: ...
    def getDisplayUnicodeExtensionKey(self, string: str, locale: java.util.Locale) -> str: ...
    def getDisplayUnicodeExtensionType(self, string: str, string2: str, locale: java.util.Locale) -> str: ...
    def getDisplayVariant(self, string: str, locale: java.util.Locale) -> str: ...

class TimeZoneNameProvider(LocaleServiceProvider):
    def getDisplayName(self, string: str, boolean: bool, int: int, locale: java.util.Locale) -> str: ...
    def getGenericDisplayName(self, string: str, int: int, locale: java.util.Locale) -> str: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("java.util.spi")``.

    AbstractResourceBundleProvider: typing.Type[AbstractResourceBundleProvider]
    CalendarDataProvider: typing.Type[CalendarDataProvider]
    CalendarNameProvider: typing.Type[CalendarNameProvider]
    CurrencyNameProvider: typing.Type[CurrencyNameProvider]
    LocaleNameProvider: typing.Type[LocaleNameProvider]
    LocaleServiceProvider: typing.Type[LocaleServiceProvider]
    ResourceBundleControlProvider: typing.Type[ResourceBundleControlProvider]
    ResourceBundleProvider: typing.Type[ResourceBundleProvider]
    TimeZoneNameProvider: typing.Type[TimeZoneNameProvider]
    ToolProvider: typing.Type[ToolProvider]



import java.lang
import scala
import scala.collection
import scala.collection.generic
import scala.collection.immutable
import scala.collection.mutable
import scala.collection.parallel
import scala.math
import scala.reflect
import scala.runtime
import scala.util
import scalan
import scorex.util.serialization
import sigmastate
import sigmastate.eval
import sigmastate.serialization
import sigmastate.utils
import sigmastate.utxo
import special.sigma
import typing



class DisabledRule:
    @staticmethod
    def canEqual(x$1: typing.Any) -> bool: ...
    @staticmethod
    def hashCode() -> int: ...
    @staticmethod
    def productArity() -> int: ...
    @staticmethod
    def productElement(x$1: int) -> typing.Any: ...
    @staticmethod
    def productIterator() -> scala.collection.Iterator[typing.Any]: ...
    @staticmethod
    def productPrefix() -> java.lang.String: ...
    @staticmethod
    def statusCode() -> int: ...
    @staticmethod
    def toString() -> java.lang.String: ...

class EnabledRule:
    @staticmethod
    def canEqual(x$1: typing.Any) -> bool: ...
    @staticmethod
    def hashCode() -> int: ...
    @staticmethod
    def productArity() -> int: ...
    @staticmethod
    def productElement(x$1: int) -> typing.Any: ...
    @staticmethod
    def productIterator() -> scala.collection.Iterator[typing.Any]: ...
    @staticmethod
    def productPrefix() -> java.lang.String: ...
    @staticmethod
    def statusCode() -> int: ...
    @staticmethod
    def toString() -> java.lang.String: ...

class RuleStatus:
    @staticmethod
    def ChangedRuleCode() -> int: ...
    @staticmethod
    def DisabledRuleCode() -> int: ...
    @staticmethod
    def EnabledRuleCode() -> int: ...
    @staticmethod
    def ReplacedRuleCode() -> int: ...
    def statusCode(self) -> int: ...

class RuleStatusSerializer:
    @staticmethod
    def FirstRuleId() -> int: ...
    @staticmethod
    def error(msg: typing.Union[java.lang.String, str]) -> scala.runtime.Nothing.: ...
    @staticmethod
    def fromBytes(bytes: typing.List[int]) -> typing.Any: ...
    @staticmethod
    def measureWrittenBytes(block: scala.Function1[sigmastate.utils.SigmaByteWriter, scala.runtime.BoxedUnit]) -> int: ...
    @staticmethod
    def parse(r: sigmastate.utils.SigmaByteReader) -> RuleStatus: ...
    @staticmethod
    def parseTry(r: scorex.util.serialization.Reader) -> scala.util.Try: ...
    @staticmethod
    def parseWithGenericReader(r: scorex.util.serialization.Reader, vs: 'SigmaValidationSettings') -> typing.Any: ...
    @staticmethod
    def serialize(status: RuleStatus, w: sigmastate.utils.SigmaByteWriter) -> None: ...
    @staticmethod
    def serializeWithGenericWriter(obj: typing.Any, w: scorex.util.serialization.Writer) -> None: ...
    @staticmethod
    def toBytes(obj: typing.Any) -> typing.List[int]: ...

class SigmaValidationSettings(scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]):
    def __init__(self): ...
    _$colon$bslash__B = typing.TypeVar('_$colon$bslash__B')  # <B>
    def $colon$bslash(self, z: _.colon.bslash__B, op: scala.Function2[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], _.colon.bslash__B, _.colon.bslash__B]) -> _.colon.bslash__B: ...
    _$div$colon__B = typing.TypeVar('_$div$colon__B')  # <B>
    def $div$colon(self, z: _.div.colon__B, op: scala.Function2[_.div.colon__B, scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], _.div.colon__B]) -> _.div.colon__B: ...
    _$plus$plus__B = typing.TypeVar('_$plus$plus__B')  # <B>
    _$plus$plus__That = typing.TypeVar('_$plus$plus__That')  # <That>
    def $plus$plus(self, that: scala.collection.GenTraversableOnce[_.plus.plus__B], bf: scala.collection.generic.CanBuildFrom[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]], _.plus.plus__B, _.plus.plus__That]) -> _.plus.plus__That: ...
    _$plus$plus$colon_0__B = typing.TypeVar('_$plus$plus$colon_0__B')  # <B>
    _$plus$plus$colon_0__That = typing.TypeVar('_$plus$plus$colon_0__That')  # <That>
    _$plus$plus$colon_1__B = typing.TypeVar('_$plus$plus$colon_1__B')  # <B>
    _$plus$plus$colon_1__That = typing.TypeVar('_$plus$plus$colon_1__That')  # <That>
    @typing.overload
    def $plus$plus$colon(self, that: scala.collection.Traversable[_.plus.plus.colon_0__B], bf: scala.collection.generic.CanBuildFrom[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]], _.plus.plus.colon_0__B, _.plus.plus.colon_0__That]) -> _.plus.plus.colon_0__That: ...
    @typing.overload
    def $plus$plus$colon(self, that: scala.collection.TraversableOnce[_.plus.plus.colon_1__B], bf: scala.collection.generic.CanBuildFrom[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]], _.plus.plus.colon_1__B, _.plus.plus.colon_1__That]) -> _.plus.plus.colon_1__That: ...
    @typing.overload
    def addString(self, b: scala.collection.mutable.StringBuilder) -> scala.collection.mutable.StringBuilder: ...
    @typing.overload
    def addString(self, b: scala.collection.mutable.StringBuilder, sep: typing.Union[java.lang.String, str]) -> scala.collection.mutable.StringBuilder: ...
    @typing.overload
    def addString(self, b: scala.collection.mutable.StringBuilder, start: typing.Union[java.lang.String, str], sep: typing.Union[java.lang.String, str], end: typing.Union[java.lang.String, str]) -> scala.collection.mutable.StringBuilder: ...
    _aggregate__B = typing.TypeVar('_aggregate__B')  # <B>
    def aggregate(self, z: scala.Function0[_aggregate__B], seqop: scala.Function2[_aggregate__B, scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], _aggregate__B], combop: scala.Function2[_aggregate__B, _aggregate__B, _aggregate__B]) -> _aggregate__B: ...
    def canEqual(self, that: typing.Any) -> bool: ...
    _collect__B = typing.TypeVar('_collect__B')  # <B>
    _collect__That = typing.TypeVar('_collect__That')  # <That>
    def collect(self, pf: scala.PartialFunction[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], _collect__B], bf: scala.collection.generic.CanBuildFrom[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]], _collect__B, _collect__That]) -> _collect__That: ...
    _collectFirst__B = typing.TypeVar('_collectFirst__B')  # <B>
    def collectFirst(self, pf: scala.PartialFunction[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], _collectFirst__B]) -> scala.Option[_collectFirst__B]: ...
    def companion(self) -> scala.collection.generic.GenericCompanion[scala.collection.Iterable]: ...
    _copyToArray_0__B = typing.TypeVar('_copyToArray_0__B')  # <B>
    _copyToArray_1__B = typing.TypeVar('_copyToArray_1__B')  # <B>
    _copyToArray_2__B = typing.TypeVar('_copyToArray_2__B')  # <B>
    @typing.overload
    def copyToArray(self, xs: typing.Any) -> None: ...
    @typing.overload
    def copyToArray(self, xs: typing.Any, start: int) -> None: ...
    @typing.overload
    def copyToArray(self, xs: typing.Any, start: int, len: int) -> None: ...
    _copyToBuffer__B = typing.TypeVar('_copyToBuffer__B')  # <B>
    def copyToBuffer(self, dest: scala.collection.mutable.Buffer[_copyToBuffer__B]) -> None: ...
    def count(self, p: scala.Function1[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], typing.Any]) -> int: ...
    def drop(self, n: int) -> typing.Any: ...
    def dropRight(self, n: int) -> typing.Any: ...
    def dropWhile(self, p: scala.Function1) -> typing.Any: ...
    def exists(self, p: scala.Function1[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], typing.Any]) -> bool: ...
    def filter(self, p: scala.Function1) -> typing.Any: ...
    def filterImpl(self, p: scala.Function1, isFlipped: bool) -> typing.Any: ...
    def filterNot(self, p: scala.Function1) -> typing.Any: ...
    def find(self, p: scala.Function1[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], typing.Any]) -> scala.Option[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]: ...
    _flatMap__B = typing.TypeVar('_flatMap__B')  # <B>
    _flatMap__That = typing.TypeVar('_flatMap__That')  # <That>
    def flatMap(self, f: scala.Function1[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], scala.collection.GenTraversableOnce[_flatMap__B]], bf: scala.collection.generic.CanBuildFrom[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]], _flatMap__B, _flatMap__That]) -> _flatMap__That: ...
    def flatten(self, asTraversable: scala.Function1) -> scala.collection.GenTraversable: ...
    _fold__A1 = typing.TypeVar('_fold__A1')  # <A1>
    def fold(self, z: _fold__A1, op: scala.Function2[_fold__A1, _fold__A1, _fold__A1]) -> _fold__A1: ...
    _foldLeft__B = typing.TypeVar('_foldLeft__B')  # <B>
    def foldLeft(self, z: _foldLeft__B, op: scala.Function2[_foldLeft__B, scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], _foldLeft__B]) -> _foldLeft__B: ...
    _foldRight__B = typing.TypeVar('_foldRight__B')  # <B>
    def foldRight(self, z: _foldRight__B, op: scala.Function2[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], _foldRight__B, _foldRight__B]) -> _foldRight__B: ...
    def forall(self, p: scala.Function1[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], typing.Any]) -> bool: ...
    _foreach__U = typing.TypeVar('_foreach__U')  # <U>
    def foreach(self, f: scala.Function1[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], _foreach__U]) -> None: ...
    _genericBuilder__B = typing.TypeVar('_genericBuilder__B')  # <B>
    def genericBuilder(self) -> scala.collection.mutable.Builder[_genericBuilder__B, scala.collection.Iterable[_genericBuilder__B]]: ...
    def get(self, id: int) -> scala.Option[scala.Tuple2['ValidationRule', RuleStatus]]: ...
    def getStatus(self, id: int) -> scala.Option[RuleStatus]: ...
    _groupBy__K = typing.TypeVar('_groupBy__K')  # <K>
    def groupBy(self, f: scala.Function1[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], _groupBy__K]) -> scala.collection.immutable.Map[_groupBy__K, scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]]: ...
    def grouped(self, size: int) -> scala.collection.Iterator[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]]: ...
    def hasDefiniteSize(self) -> bool: ...
    def head(self) -> typing.Any: ...
    def headOption(self) -> scala.Option[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]: ...
    def init(self) -> typing.Any: ...
    def inits(self) -> scala.collection.Iterator[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]]: ...
    def isEmpty(self) -> bool: ...
    @typing.overload
    def isSoftFork(self, ve: 'ValidationException') -> bool: ...
    @typing.overload
    def isSoftFork(self, ruleId: int, ve: 'ValidationException') -> bool: ...
    def isTraversableAgain(self) -> bool: ...
    def last(self) -> typing.Any: ...
    def lastOption(self) -> scala.Option[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]: ...
    _map__B = typing.TypeVar('_map__B')  # <B>
    _map__That = typing.TypeVar('_map__That')  # <That>
    def map(self, f: scala.Function1[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], _map__B], bf: scala.collection.generic.CanBuildFrom[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]], _map__B, _map__That]) -> _map__That: ...
    def max(self, cmp: scala.math.Ordering) -> typing.Any: ...
    def maxBy(self, f: scala.Function1, cmp: scala.math.Ordering) -> typing.Any: ...
    def min(self, cmp: scala.math.Ordering) -> typing.Any: ...
    def minBy(self, f: scala.Function1, cmp: scala.math.Ordering) -> typing.Any: ...
    @typing.overload
    def mkString(self) -> java.lang.String: ...
    @typing.overload
    def mkString(self, sep: typing.Union[java.lang.String, str]) -> java.lang.String: ...
    @typing.overload
    def mkString(self, start: typing.Union[java.lang.String, str], sep: typing.Union[java.lang.String, str], end: typing.Union[java.lang.String, str]) -> java.lang.String: ...
    def newBuilder(self) -> scala.collection.mutable.Builder[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]]: ...
    def nonEmpty(self) -> bool: ...
    def par(self) -> scala.collection.Parallel: ...
    def parCombiner(self) -> scala.collection.parallel.Combiner[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], scala.collection.parallel.ParIterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]]: ...
    def partition(self, p: scala.Function1[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], typing.Any]) -> scala.Tuple2[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]], scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]]: ...
    _product__B = typing.TypeVar('_product__B')  # <B>
    def product(self, num: scala.math.Numeric[_product__B]) -> _product__B: ...
    _reduce__A1 = typing.TypeVar('_reduce__A1')  # <A1>
    def reduce(self, op: scala.Function2[_reduce__A1, _reduce__A1, _reduce__A1]) -> _reduce__A1: ...
    _reduceLeft__B = typing.TypeVar('_reduceLeft__B')  # <B>
    def reduceLeft(self, op: scala.Function2[_reduceLeft__B, scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], _reduceLeft__B]) -> _reduceLeft__B: ...
    _reduceLeftOption__B = typing.TypeVar('_reduceLeftOption__B')  # <B>
    def reduceLeftOption(self, op: scala.Function2[_reduceLeftOption__B, scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], _reduceLeftOption__B]) -> scala.Option[_reduceLeftOption__B]: ...
    _reduceOption__A1 = typing.TypeVar('_reduceOption__A1')  # <A1>
    def reduceOption(self, op: scala.Function2[_reduceOption__A1, _reduceOption__A1, _reduceOption__A1]) -> scala.Option[_reduceOption__A1]: ...
    _reduceRight__B = typing.TypeVar('_reduceRight__B')  # <B>
    def reduceRight(self, op: scala.Function2[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], _reduceRight__B, _reduceRight__B]) -> _reduceRight__B: ...
    _reduceRightOption__B = typing.TypeVar('_reduceRightOption__B')  # <B>
    def reduceRightOption(self, op: scala.Function2[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], _reduceRightOption__B, _reduceRightOption__B]) -> scala.Option[_reduceRightOption__B]: ...
    def repr(self) -> typing.Any: ...
    def reversed(self) -> scala.collection.immutable.List[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]: ...
    _sameElements__B = typing.TypeVar('_sameElements__B')  # <B>
    def sameElements(self, that: scala.collection.GenIterable[_sameElements__B]) -> bool: ...
    _scan__B = typing.TypeVar('_scan__B')  # <B>
    _scan__That = typing.TypeVar('_scan__That')  # <That>
    def scan(self, z: _scan__B, op: scala.Function2[_scan__B, _scan__B, _scan__B], cbf: scala.collection.generic.CanBuildFrom[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]], _scan__B, _scan__That]) -> _scan__That: ...
    _scanLeft__B = typing.TypeVar('_scanLeft__B')  # <B>
    _scanLeft__That = typing.TypeVar('_scanLeft__That')  # <That>
    def scanLeft(self, z: _scanLeft__B, op: scala.Function2[_scanLeft__B, scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], _scanLeft__B], bf: scala.collection.generic.CanBuildFrom[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]], _scanLeft__B, _scanLeft__That]) -> _scanLeft__That: ...
    _scanRight__B = typing.TypeVar('_scanRight__B')  # <B>
    _scanRight__That = typing.TypeVar('_scanRight__That')  # <That>
    def scanRight(self, z: _scanRight__B, op: scala.Function2[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], _scanRight__B, _scanRight__B], bf: scala.collection.generic.CanBuildFrom[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]], _scanRight__B, _scanRight__That]) -> _scanRight__That: ...
    def seq(self) -> scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]: ...
    def size(self) -> int: ...
    def sizeHintIfCheap(self) -> int: ...
    def slice(self, from_: int, until: int) -> typing.Any: ...
    def sliceWithKnownBound(self, from_: int, until: int) -> typing.Any: ...
    def sliceWithKnownDelta(self, from_: int, until: int, delta: int) -> typing.Any: ...
    @typing.overload
    def sliding(self, size: int) -> scala.collection.Iterator[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]]: ...
    @typing.overload
    def sliding(self, size: int, step: int) -> scala.collection.Iterator[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]]: ...
    def span(self, p: scala.Function1[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], typing.Any]) -> scala.Tuple2[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]], scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]]: ...
    def splitAt(self, n: int) -> scala.Tuple2[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]], scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]]: ...
    def stringPrefix(self) -> java.lang.String: ...
    _sum__B = typing.TypeVar('_sum__B')  # <B>
    def sum(self, num: scala.math.Numeric[_sum__B]) -> _sum__B: ...
    def tail(self) -> typing.Any: ...
    def tails(self) -> scala.collection.Iterator[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]]: ...
    def take(self, n: int) -> typing.Any: ...
    def takeRight(self, n: int) -> typing.Any: ...
    def takeWhile(self, p: scala.Function1) -> typing.Any: ...
    def thisCollection(self) -> scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]: ...
    _to__Col = typing.TypeVar('_to__Col')  # <Col>
    def to(self, cbf: scala.collection.generic.CanBuildFrom[scala.runtime.Nothing., scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], _to__Col]) -> _to__Col: ...
    _toArray__B = typing.TypeVar('_toArray__B')  # <B>
    def toArray(self, evidence$1: scala.reflect.ClassTag[_toArray__B]) -> typing.Any: ...
    _toBuffer__B = typing.TypeVar('_toBuffer__B')  # <B>
    def toBuffer(self) -> scala.collection.mutable.Buffer[_toBuffer__B]: ...
    def toCollection(self, repr: typing.Any) -> scala.collection.Iterable: ...
    def toIndexedSeq(self) -> scala.collection.immutable.IndexedSeq[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]: ...
    def toIterable(self) -> scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]: ...
    def toIterator(self) -> scala.collection.Iterator[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]: ...
    def toList(self) -> scala.collection.immutable.List[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]: ...
    _toMap__T = typing.TypeVar('_toMap__T')  # <T>
    _toMap__U = typing.TypeVar('_toMap__U')  # <U>
    def toMap(self, ev: scala.Predef..less.colon.less[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], scala.Tuple2[_toMap__T, _toMap__U]]) -> scala.collection.immutable.Map[_toMap__T, _toMap__U]: ...
    def toSeq(self) -> scala.collection.Seq[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]: ...
    _toSet__B = typing.TypeVar('_toSet__B')  # <B>
    def toSet(self) -> scala.collection.immutable.Set[_toSet__B]: ...
    def toStream(self) -> scala.collection.immutable.Stream[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]: ...
    def toString(self) -> java.lang.String: ...
    def toTraversable(self) -> scala.collection.Traversable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]: ...
    def toVector(self) -> scala.collection.immutable.Vector[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]: ...
    def transpose(self, asTraversable: scala.Function1) -> scala.collection.GenTraversable: ...
    _unzip__A1 = typing.TypeVar('_unzip__A1')  # <A1>
    _unzip__A2 = typing.TypeVar('_unzip__A2')  # <A2>
    def unzip(self, asPair: scala.Function1[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], scala.Tuple2[_unzip__A1, _unzip__A2]]) -> scala.Tuple2[scala.collection.Iterable[_unzip__A1], scala.collection.Iterable[_unzip__A2]]: ...
    _unzip3__A1 = typing.TypeVar('_unzip3__A1')  # <A1>
    _unzip3__A2 = typing.TypeVar('_unzip3__A2')  # <A2>
    _unzip3__A3 = typing.TypeVar('_unzip3__A3')  # <A3>
    def unzip3(self, asTriple: scala.Function1[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], scala.Tuple3[_unzip3__A1, _unzip3__A2, _unzip3__A3]]) -> scala.Tuple3[scala.collection.Iterable[_unzip3__A1], scala.collection.Iterable[_unzip3__A2], scala.collection.Iterable[_unzip3__A3]]: ...
    def updated(self, id: int, newStatus: RuleStatus) -> 'SigmaValidationSettings': ...
    @typing.overload
    def view(self) -> scala.collection.IterableView[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]]: ...
    @typing.overload
    def view(self, from_: int, until: int) -> scala.collection.IterableView[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]]: ...
    def withFilter(self, p: scala.Function1[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], typing.Any]) -> scala.collection.generic.FilterMonadic[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]], scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]]: ...
    _zip__A1 = typing.TypeVar('_zip__A1')  # <A1>
    _zip__B = typing.TypeVar('_zip__B')  # <B>
    _zip__That = typing.TypeVar('_zip__That')  # <That>
    def zip(self, that: scala.collection.GenIterable[_zip__B], bf: scala.collection.generic.CanBuildFrom[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]], scala.Tuple2[_zip__A1, _zip__B], _zip__That]) -> _zip__That: ...
    _zipAll__B = typing.TypeVar('_zipAll__B')  # <B>
    _zipAll__A1 = typing.TypeVar('_zipAll__A1')  # <A1>
    _zipAll__That = typing.TypeVar('_zipAll__That')  # <That>
    def zipAll(self, that: scala.collection.GenIterable[_zipAll__B], thisElem: _zipAll__A1, thatElem: _zipAll__B, bf: scala.collection.generic.CanBuildFrom[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]], scala.Tuple2[_zipAll__A1, _zipAll__B], _zipAll__That]) -> _zipAll__That: ...
    _zipWithIndex__A1 = typing.TypeVar('_zipWithIndex__A1')  # <A1>
    _zipWithIndex__That = typing.TypeVar('_zipWithIndex__That')  # <That>
    def zipWithIndex(self, bf: scala.collection.generic.CanBuildFrom[scala.collection.Iterable[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]], scala.Tuple2[_zipWithIndex__A1, typing.Any], _zipWithIndex__That]) -> _zipWithIndex__That: ...

class SigmaValidationSettingsSerializer:
    @staticmethod
    def error(msg: typing.Union[java.lang.String, str]) -> scala.runtime.Nothing.: ...
    @staticmethod
    def fromBytes(bytes: typing.List[int]) -> typing.Any: ...
    @staticmethod
    def parse(r: sigmastate.utils.SigmaByteReader) -> SigmaValidationSettings: ...
    @staticmethod
    def parseTry(r: scorex.util.serialization.Reader) -> scala.util.Try: ...
    @staticmethod
    def parseWithGenericReader(r: scorex.util.serialization.Reader, vs: SigmaValidationSettings) -> typing.Any: ...
    @staticmethod
    def serialize(settings: SigmaValidationSettings, w: sigmastate.utils.SigmaByteWriter) -> None: ...
    @staticmethod
    def serializeWithGenericWriter(obj: typing.Any, w: scorex.util.serialization.Writer) -> None: ...
    @staticmethod
    def toBytes(obj: typing.Any) -> typing.List[int]: ...

class SoftForkChecker:
    @staticmethod
    def $init$($this: 'SoftForkChecker') -> None: ...
    def isSoftFork(self, vs: SigmaValidationSettings, ruleId: int, status: RuleStatus, args: scala.collection.Seq[typing.Any]) -> bool: ...

class ValidationException(java.lang.Exception, scala.Product, scala.Serializable):
    def __init__(self, message: typing.Union[java.lang.String, str], rule: 'ValidationRule', args: scala.collection.Seq[typing.Any], cause: scala.Option[java.lang.Throwable]): ...
    @staticmethod
    def $lessinit$greater$default$4() -> scala.Option[java.lang.Throwable]: ...
    @staticmethod
    def apply(message: typing.Union[java.lang.String, str], rule: 'ValidationRule', args: scala.collection.Seq[typing.Any], cause: scala.Option[java.lang.Throwable]) -> 'ValidationException': ...
    @staticmethod
    def apply$default$4() -> scala.Option[java.lang.Throwable]: ...
    def args(self) -> scala.collection.Seq[typing.Any]: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    def cause(self) -> scala.Option[java.lang.Throwable]: ...
    def copy(self, message: typing.Union[java.lang.String, str], rule: 'ValidationRule', args: scala.collection.Seq[typing.Any], cause: scala.Option[java.lang.Throwable]) -> 'ValidationException': ...
    def copy$default$1(self) -> java.lang.String: ...
    def copy$default$2(self) -> 'ValidationRule': ...
    def copy$default$3(self) -> scala.collection.Seq[typing.Any]: ...
    def copy$default$4(self) -> scala.Option[java.lang.Throwable]: ...
    @staticmethod
    def curried() -> scala.Function1[java.lang.String, scala.Function1['ValidationRule', scala.Function1[scala.collection.Seq[typing.Any], scala.Function1[scala.Option[java.lang.Throwable], 'ValidationException']]]]: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def fillInStackTrace(self) -> java.lang.Throwable: ...
    def hashCode(self) -> int: ...
    def message(self) -> java.lang.String: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def rule(self) -> 'ValidationRule': ...
    @staticmethod
    def tupled() -> scala.Function1[scala.Tuple4[java.lang.String, 'ValidationRule', scala.collection.Seq[typing.Any], scala.Option[java.lang.Throwable]], 'ValidationException']: ...
    @staticmethod
    def unapply(x$0: 'ValidationException') -> scala.Option[scala.Tuple4[java.lang.String, 'ValidationRule', scala.collection.Seq[typing.Any], scala.Option[java.lang.Throwable]]]: ...

class ChangedRule(RuleStatus, scala.Product, scala.Serializable):
    def __init__(self, newValue: typing.List[int]): ...
    _andThen__A = typing.TypeVar('_andThen__A')  # <A>
    @staticmethod
    def andThen(g: scala.Function1['ChangedRule', _andThen__A]) -> scala.Function1[typing.List[int], _andThen__A]: ...
    @staticmethod
    def apply(newValue: typing.List[int]) -> 'ChangedRule': ...
    def canEqual(self, that: typing.Any) -> bool: ...
    _compose__A = typing.TypeVar('_compose__A')  # <A>
    @staticmethod
    def compose(g: scala.Function1[_compose__A, typing.List[int]]) -> scala.Function1[_compose__A, 'ChangedRule']: ...
    def copy(self, newValue: typing.List[int]) -> 'ChangedRule': ...
    def copy$default$1(self) -> typing.List[int]: ...
    def equals(self, obj: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def newValue(self) -> typing.List[int]: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def statusCode(self) -> int: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'ChangedRule') -> scala.Option[typing.List[int]]: ...

class MapSigmaValidationSettings(SigmaValidationSettings):
    def __init__(self, map: scala.collection.immutable.Map[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]): ...
    def canEqual(self, that: typing.Any) -> bool: ...
    def equals(self, obj: typing.Any) -> bool: ...
    def get(self, id: int) -> scala.Option[scala.Tuple2['ValidationRule', RuleStatus]]: ...
    def getStatus(self, id: int) -> scala.Option[RuleStatus]: ...
    def hashCode(self) -> int: ...
    def iterator(self) -> scala.collection.Iterator[scala.Tuple2[typing.Any, scala.Tuple2['ValidationRule', RuleStatus]]]: ...
    def updated(self, id: int, newStatus: RuleStatus) -> 'MapSigmaValidationSettings': ...

class ReplacedRule(RuleStatus, scala.Product, scala.Serializable):
    def __init__(self, newRuleId: int): ...
    _andThen__A = typing.TypeVar('_andThen__A')  # <A>
    @staticmethod
    def andThen(g: scala.Function1['ReplacedRule', _andThen__A]) -> scala.Function1[typing.Any, _andThen__A]: ...
    @staticmethod
    def apply(newRuleId: int) -> 'ReplacedRule': ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _compose__A = typing.TypeVar('_compose__A')  # <A>
    @staticmethod
    def compose(g: scala.Function1[_compose__A, typing.Any]) -> scala.Function1[_compose__A, 'ReplacedRule']: ...
    def copy(self, newRuleId: int) -> 'ReplacedRule': ...
    def copy$default$1(self) -> int: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def newRuleId(self) -> int: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def statusCode(self) -> int: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'ReplacedRule') -> scala.Option[typing.Any]: ...

class SoftForkWhenCodeAdded(SoftForkChecker):
    @staticmethod
    def $init$($this: 'SoftForkWhenCodeAdded') -> None: ...
    def isSoftFork(self, vs: SigmaValidationSettings, ruleId: int, status: RuleStatus, args: scala.collection.Seq[typing.Any]) -> bool: ...

class SoftForkWhenReplaced(SoftForkChecker):
    @staticmethod
    def $init$($this: 'SoftForkWhenReplaced') -> None: ...
    def isSoftFork(self, vs: SigmaValidationSettings, ruleId: int, status: RuleStatus, args: scala.collection.Seq[typing.Any]) -> bool: ...

class ValidationRule(SoftForkChecker, scala.Product, scala.Serializable):
    def __init__(self, id: int, description: typing.Union[java.lang.String, str]): ...
    @staticmethod
    def apply(id: int, description: typing.Union[java.lang.String, str]) -> 'ValidationRule': ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    def checkRule(self) -> None: ...
    def copy(self, id: int, description: typing.Union[java.lang.String, str]) -> 'ValidationRule': ...
    def copy$default$1(self) -> int: ...
    def copy$default$2(self) -> java.lang.String: ...
    @staticmethod
    def curried() -> scala.Function1[typing.Any, scala.Function1[java.lang.String, 'ValidationRule']]: ...
    def description(self) -> java.lang.String: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def id(self) -> int: ...
    def isSoftFork(self, vs: SigmaValidationSettings, ruleId: int, status: RuleStatus, args: scala.collection.Seq[typing.Any]) -> bool: ...
    def org$ergoplatform$validation$ValidationRule$$_checked(self) -> bool: ...
    def org$ergoplatform$validation$ValidationRule$$_checked_$eq(self, x$1: bool) -> None: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def throwValidationException(self, cause: java.lang.Throwable, args: scala.collection.Seq[typing.Any]) -> scala.runtime.Nothing.: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def tupled() -> scala.Function1[scala.Tuple2[typing.Any, java.lang.String], 'ValidationRule']: ...
    @staticmethod
    def unapply(x$0: 'ValidationRule') -> scala.Option[scala.Tuple2[typing.Any, java.lang.String]]: ...

class ValidationRules:
    @staticmethod
    def FirstRuleId() -> int: ...
    @staticmethod
    def currentSettings() -> SigmaValidationSettings: ...
    @staticmethod
    def ruleSpecs() -> scala.collection.Seq[ValidationRule]: ...
    _trySoftForkable__T = typing.TypeVar('_trySoftForkable__T')  # <T>
    @staticmethod
    def trySoftForkable(whenSoftFork: scala.Function0[_trySoftForkable__T], block: scala.Function0[_trySoftForkable__T], vs: SigmaValidationSettings) -> _trySoftForkable__T: ...
    class CheckAndGetMethod$(ValidationRule):
        MODULE$: typing.ClassVar['ValidationRules.CheckAndGetMethod.'] = ...
        def __init__(self): ...
        _apply_0__T = typing.TypeVar('_apply_0__T')  # <T>
        @typing.overload
        def apply(self, objType: sigmastate.STypeCompanion, methodId: int) -> sigmastate.SMethod: ...
        @typing.overload
        @staticmethod
        def apply(id: int, description: typing.Union[java.lang.String, str]) -> ValidationRule: ...
        def isSoftFork(self, vs: SigmaValidationSettings, ruleId: int, status: RuleStatus, args: scala.collection.Seq[typing.Any]) -> bool: ...
    class CheckCalcFunc$(ValidationRule):
        MODULE$: typing.ClassVar['ValidationRules.CheckCalcFunc.'] = ...
        def __init__(self): ...
        _apply_0__Ctx = typing.TypeVar('_apply_0__Ctx', bound=sigmastate.eval.IRContext)  # <Ctx>
        _apply_0__T = typing.TypeVar('_apply_0__T')  # <T>
        @typing.overload
        def apply(self, ctx: _apply_0__Ctx, calcF: scalan.Base.Ref[scala.Function1[special.sigma.SigmaDsl.Context, typing.Any]]) -> None: ...
        @typing.overload
        @staticmethod
        def apply(id: int, description: typing.Union[java.lang.String, str]) -> ValidationRule: ...
    class CheckCostFunc$(ValidationRule):
        MODULE$: typing.ClassVar['ValidationRules.CheckCostFunc.'] = ...
        def __init__(self): ...
        _apply_0__Ctx = typing.TypeVar('_apply_0__Ctx', bound=sigmastate.eval.IRContext)  # <Ctx>
        _apply_0__T = typing.TypeVar('_apply_0__T')  # <T>
        @typing.overload
        def apply(self, ctx: _apply_0__Ctx, costF: scalan.Base.Ref[scala.Function1[typing.Any, typing.Any]]) -> None: ...
        @typing.overload
        @staticmethod
        def apply(id: int, description: typing.Union[java.lang.String, str]) -> ValidationRule: ...
    class CheckCostFuncOperation$(ValidationRule, SoftForkWhenCodeAdded):
        MODULE$: typing.ClassVar['ValidationRules.CheckCostFuncOperation.'] = ...
        def __init__(self): ...
        _apply_0__Ctx = typing.TypeVar('_apply_0__Ctx', bound=sigmastate.eval.IRContext)  # <Ctx>
        _apply_0__T = typing.TypeVar('_apply_0__T')  # <T>
        @typing.overload
        def apply(self, ctx: _apply_0__Ctx, opCode: int) -> None: ...
        @typing.overload
        @staticmethod
        def apply(id: int, description: typing.Union[java.lang.String, str]) -> ValidationRule: ...
        def decodeVLQUShort(self, bytes: typing.List[int]) -> scala.collection.Seq[typing.Any]: ...
        def encodeVLQUShort(self, opCodes: scala.collection.Seq[typing.Any]) -> typing.List[int]: ...
        def isSoftFork(self, vs: SigmaValidationSettings, ruleId: int, status: RuleStatus, args: scala.collection.Seq[typing.Any]) -> bool: ...
    class CheckDeserializedScriptIsSigmaProp$(ValidationRule):
        MODULE$: typing.ClassVar['ValidationRules.CheckDeserializedScriptIsSigmaProp.'] = ...
        def __init__(self): ...
        _apply_0__T = typing.TypeVar('_apply_0__T')  # <T>
        @typing.overload
        def apply(self, root: sigmastate.Values.Value[sigmastate.SType]) -> None: ...
        @typing.overload
        @staticmethod
        def apply(id: int, description: typing.Union[java.lang.String, str]) -> ValidationRule: ...
    class CheckDeserializedScriptType$(ValidationRule):
        MODULE$: typing.ClassVar['ValidationRules.CheckDeserializedScriptType.'] = ...
        def __init__(self): ...
        _apply_0__T = typing.TypeVar('_apply_0__T')  # <T>
        @typing.overload
        def apply(self, d: sigmastate.utxo.DeserializeContext[typing.Any], script: sigmastate.Values.Value[sigmastate.SType]) -> None: ...
        @typing.overload
        @staticmethod
        def apply(id: int, description: typing.Union[java.lang.String, str]) -> ValidationRule: ...
    class CheckHeaderSizeBit$(ValidationRule, SoftForkWhenReplaced):
        MODULE$: typing.ClassVar['ValidationRules.CheckHeaderSizeBit.'] = ...
        def __init__(self): ...
        @typing.overload
        def apply(self, header: int) -> None: ...
        @typing.overload
        @staticmethod
        def apply(id: int, description: typing.Union[java.lang.String, str]) -> ValidationRule: ...
        def isSoftFork(self, vs: SigmaValidationSettings, ruleId: int, status: RuleStatus, args: scala.collection.Seq[typing.Any]) -> bool: ...
    class CheckIsSupportedIndexExpression$(ValidationRule):
        MODULE$: typing.ClassVar['ValidationRules.CheckIsSupportedIndexExpression.'] = ...
        def __init__(self): ...
        _apply_0__Ctx = typing.TypeVar('_apply_0__Ctx', bound=sigmastate.eval.IRContext)  # <Ctx>
        _apply_0__T = typing.TypeVar('_apply_0__T')  # <T>
        @typing.overload
        def apply(self, ctx: _apply_0__Ctx, coll: sigmastate.Values.Value[sigmastate.SCollection[typing.Any]], i: sigmastate.Values.Value[sigmastate.SInt.], iSym: scalan.Base.Ref[typing.Any]) -> None: ...
        @typing.overload
        @staticmethod
        def apply(id: int, description: typing.Union[java.lang.String, str]) -> ValidationRule: ...
    class CheckLoopLevelInCostFunction$(ValidationRule, SoftForkWhenReplaced):
        MODULE$: typing.ClassVar['ValidationRules.CheckLoopLevelInCostFunction.'] = ...
        def __init__(self): ...
        @typing.overload
        def apply(self, level: int) -> None: ...
        @typing.overload
        @staticmethod
        def apply(id: int, description: typing.Union[java.lang.String, str]) -> ValidationRule: ...
        def isSoftFork(self, vs: SigmaValidationSettings, ruleId: int, status: RuleStatus, args: scala.collection.Seq[typing.Any]) -> bool: ...
    class CheckPositionLimit$(ValidationRule, SoftForkWhenReplaced):
        MODULE$: typing.ClassVar['ValidationRules.CheckPositionLimit.'] = ...
        def __init__(self): ...
        def isSoftFork(self, vs: SigmaValidationSettings, ruleId: int, status: RuleStatus, args: scala.collection.Seq[typing.Any]) -> bool: ...
    class CheckPrimitiveTypeCode$(ValidationRule, SoftForkWhenCodeAdded):
        MODULE$: typing.ClassVar['ValidationRules.CheckPrimitiveTypeCode.'] = ...
        def __init__(self): ...
        _apply_0__T = typing.TypeVar('_apply_0__T')  # <T>
        @typing.overload
        def apply(self, code: int) -> None: ...
        @typing.overload
        @staticmethod
        def apply(id: int, description: typing.Union[java.lang.String, str]) -> ValidationRule: ...
        def isSoftFork(self, vs: SigmaValidationSettings, ruleId: int, status: RuleStatus, args: scala.collection.Seq[typing.Any]) -> bool: ...
    class CheckSerializableTypeCode$(ValidationRule, SoftForkWhenReplaced):
        MODULE$: typing.ClassVar['ValidationRules.CheckSerializableTypeCode.'] = ...
        def __init__(self): ...
        _apply_0__T = typing.TypeVar('_apply_0__T')  # <T>
        @typing.overload
        def apply(self, typeCode: int) -> None: ...
        @typing.overload
        @staticmethod
        def apply(id: int, description: typing.Union[java.lang.String, str]) -> ValidationRule: ...
        def isSoftFork(self, vs: SigmaValidationSettings, ruleId: int, status: RuleStatus, args: scala.collection.Seq[typing.Any]) -> bool: ...
    class CheckTupleType$(ValidationRule, SoftForkWhenReplaced):
        MODULE$: typing.ClassVar['ValidationRules.CheckTupleType.'] = ...
        def __init__(self): ...
        _apply_0__Ctx = typing.TypeVar('_apply_0__Ctx', bound=sigmastate.eval.IRContext)  # <Ctx>
        _apply_0__T = typing.TypeVar('_apply_0__T')  # <T>
        @typing.overload
        def apply(self, ctx: _apply_0__Ctx, e: scalan.TypeDescs.Elem[typing.Any]) -> None: ...
        @typing.overload
        @staticmethod
        def apply(id: int, description: typing.Union[java.lang.String, str]) -> ValidationRule: ...
        def isSoftFork(self, vs: SigmaValidationSettings, ruleId: int, status: RuleStatus, args: scala.collection.Seq[typing.Any]) -> bool: ...
    class CheckTypeCode$(ValidationRule, SoftForkWhenCodeAdded):
        MODULE$: typing.ClassVar['ValidationRules.CheckTypeCode.'] = ...
        def __init__(self): ...
        _apply_0__T = typing.TypeVar('_apply_0__T')  # <T>
        @typing.overload
        def apply(self, typeCode: int) -> None: ...
        @typing.overload
        @staticmethod
        def apply(id: int, description: typing.Union[java.lang.String, str]) -> ValidationRule: ...
        def isSoftFork(self, vs: SigmaValidationSettings, ruleId: int, status: RuleStatus, args: scala.collection.Seq[typing.Any]) -> bool: ...
    class CheckTypeWithMethods$(ValidationRule, SoftForkWhenCodeAdded):
        MODULE$: typing.ClassVar['ValidationRules.CheckTypeWithMethods.'] = ...
        def __init__(self): ...
        _apply_0__T = typing.TypeVar('_apply_0__T')  # <T>
        @typing.overload
        def apply(self, typeCode: int, cond: bool) -> None: ...
        @typing.overload
        @staticmethod
        def apply(id: int, description: typing.Union[java.lang.String, str]) -> ValidationRule: ...
        def isSoftFork(self, vs: SigmaValidationSettings, ruleId: int, status: RuleStatus, args: scala.collection.Seq[typing.Any]) -> bool: ...
    class CheckValidOpCode$(ValidationRule, SoftForkWhenCodeAdded):
        MODULE$: typing.ClassVar['ValidationRules.CheckValidOpCode.'] = ...
        def __init__(self): ...
        _apply_0__T = typing.TypeVar('_apply_0__T')  # <T>
        @typing.overload
        def apply(self, ser: sigmastate.serialization.ValueSerializer[typing.Any], opCode: int) -> None: ...
        @typing.overload
        @staticmethod
        def apply(id: int, description: typing.Union[java.lang.String, str]) -> ValidationRule: ...
        def isSoftFork(self, vs: SigmaValidationSettings, ruleId: int, status: RuleStatus, args: scala.collection.Seq[typing.Any]) -> bool: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("org.ergoplatform.validation")``.

    ChangedRule: typing.Type[ChangedRule]
    DisabledRule: typing.Type[DisabledRule]
    EnabledRule: typing.Type[EnabledRule]
    MapSigmaValidationSettings: typing.Type[MapSigmaValidationSettings]
    ReplacedRule: typing.Type[ReplacedRule]
    RuleStatus: typing.Type[RuleStatus]
    RuleStatusSerializer: typing.Type[RuleStatusSerializer]
    SigmaValidationSettings: typing.Type[SigmaValidationSettings]
    SigmaValidationSettingsSerializer: typing.Type[SigmaValidationSettingsSerializer]
    SoftForkChecker: typing.Type[SoftForkChecker]
    SoftForkWhenCodeAdded: typing.Type[SoftForkWhenCodeAdded]
    SoftForkWhenReplaced: typing.Type[SoftForkWhenReplaced]
    ValidationException: typing.Type[ValidationException]
    ValidationRule: typing.Type[ValidationRule]
    ValidationRules: typing.Type[ValidationRules]



import scorex.crypto.authds
import scorex.crypto.encode
import scorex.crypto.hash
import scorex.crypto.signatures
import typing


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("scorex.crypto")``.

    authds: scorex.crypto.authds.__module_protocol__
    encode: scorex.crypto.encode.__module_protocol__
    hash: scorex.crypto.hash.__module_protocol__
    signatures: scorex.crypto.signatures.__module_protocol__
